@@ -20,19 +20,27 @@
  *             Chris Lord  <chris@linux.intel.com>
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "mx-application.h"
 
 #include "mx-private.h"
+#include "mx-settings.h"
 #include "mx-window.h"
 
 #include <glib/gi18n-lib.h>
-#include <clutter/x11/clutter-x11.h>
 
 #ifdef HAVE_STARTUP_NOTIFICATION
+
 #  define SN_API_NOT_YET_FROZEN
 #  include <libsn/sn.h>
+
+#  ifdef HAVE_X11
+#    include <clutter/x11/clutter-x11.h>
+#  endif
+
 #endif
 
 #ifdef HAVE_DBUS
@@ -46,18 +54,6 @@ G_DEFINE_TYPE (MxApplication, mx_application, G_TYPE_OBJECT)
 #define APPLICATION_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MX_TYPE_APPLICATION, MxApplicationPrivate))
 
-static gchar *mx_application_atoms[] =
-{
-  "_NET_SUPPORTING_WM_CHECK",
-  "_MOBLIN"
-};
-
-enum
-{
-  MX_ATOM_NET_SUPPORTING_WM_CHECK,
-  MX_ATOM_MOBLIN
-};
-
 struct _MxApplicationPrivate
 {
   GList              *windows;
@@ -78,10 +74,6 @@ struct _MxApplicationPrivate
 #endif
 
   GHashTable         *actions;
-
-  Atom                atoms[G_N_ELEMENTS(mx_application_atoms)];
-  Window             *wm_window;
-  gboolean            small_screen;
 };
 
 enum
@@ -103,11 +95,6 @@ static guint signals[LAST_SIGNAL] = { 0, };
 
 static MxApplication *app_singleton = NULL;
 
-static ClutterX11FilterReturn
-mx_application_x11_filter_func (XEvent       *xevent,
-                                ClutterEvent *cevent,
-                                gpointer      data);
-
 static GObject*
 mx_application_constructor (GType                  type,
                             guint                  n_construct_params,
@@ -209,9 +196,33 @@ mx_application_actions_changed_cb (DBusGProxy *proxy,
 }
 #endif
 
+static void
+mx_application_notify_small_screen_cb (MxSettings    *settings,
+                                       GParamSpec    *pspec,
+                                       MxApplication *self)
+{
+  MxApplicationPrivate *priv = self->priv;
+
+  /* Reflect small-screen mode in the first added window of the
+   * application.
+   *
+   * FIXME: This should probably be optional.
+   */
+  if (priv->windows)
+    {
+      gboolean small_screen = FALSE;
+      MxWindow *window = g_list_last (priv->windows)->data;
+
+      g_object_get (G_OBJECT (settings), "small-screen", &small_screen, NULL);
+      mx_window_set_small_screen (window, small_screen);
+    }
+}
+
 static void
 mx_application_constructed (GObject *object)
 {
+  MxSettings *settings;
+
   MxApplication *self = MX_APPLICATION (object);
   MxApplicationPrivate *priv = self->priv;
   gboolean success = FALSE;
@@ -313,6 +324,11 @@ mx_application_constructed (GObject *object)
                             self);
       mx_application_add_action (self, raise_action);
     }
+
+  settings = mx_settings_get_default ();
+  if (settings)
+    g_signal_connect (settings, "notify::small-screen",
+                      G_CALLBACK (mx_application_notify_small_screen_cb), self);
 }
 
 static void
@@ -432,8 +448,6 @@ mx_application_dispose (GObject *object)
 {
   MxApplicationPrivate *priv = MX_APPLICATION (object)->priv;
 
-  clutter_x11_remove_filter (mx_application_x11_filter_func, object);
-
 #ifdef HAVE_DBUS
   if (priv->proxy)
     {
@@ -470,9 +484,6 @@ mx_application_finalize (GObject *object)
 
   g_free (priv->name);
 
-  if (priv->wm_window)
-    XFree (priv->wm_window);
-
   G_OBJECT_CLASS (mx_application_parent_class)->finalize (object);
 }
 
@@ -495,55 +506,12 @@ mx_application_default_create_window (MxApplication *application)
 static void
 mx_application_default_raise (MxApplication *application)
 {
-  Window window;
-  Display *display;
-  guint32 timestamp;
-  ClutterStage *stage;
-  XClientMessageEvent xclient;
-
   MxApplicationPrivate *priv = application->priv;
 
   if (!priv->windows)
     return;
 
-  stage =
-    mx_window_get_clutter_stage ((MxWindow *)g_list_last (priv->windows)->data);
-
-  /* As with all these arcane, poorly documented X11 things, learnt
-   * how to do this from reading GTK/GDK code.
-   */
-  display = clutter_x11_get_default_display ();
-  window = clutter_x11_get_stage_window (stage);
-  XRaiseWindow (display, window);
-
-  /* These two calls may not be necessary */
-  timestamp = 0x7FFFFFFF;
-  XChangeProperty (display,
-                   window,
-                   XInternAtom (display, "_NET_WM_USER_TIME", False),
-                   XA_CARDINAL,
-                   32,
-                   PropModeReplace,
-                   (guchar *)&timestamp,
-                   1);
-  XMapWindow (display, window);
-
-  memset (&xclient, 0, sizeof (xclient));
-  xclient.type = ClientMessage;
-  xclient.window = window;
-  xclient.message_type = XInternAtom (display, "_NET_ACTIVE_WINDOW", False);
-  xclient.format = 32;
-  xclient.data.l[0] = 1;
-  xclient.data.l[1] = timestamp;
-  xclient.data.l[2] = None;
-  xclient.data.l[3] = 0;
-  xclient.data.l[4] = 0;
-
-  XSendEvent (display,
-              clutter_x11_get_root_window (),
-              False,
-              SubstructureRedirectMask | SubstructureNotifyMask,
-              (XEvent *)&xclient);
+  mx_window_present (MX_WINDOW (g_list_last (priv->windows)->data));
 }
 
 
@@ -618,7 +586,7 @@ mx_application_window_destroy_cb (MxWindow      *window,
     }
 }
 
-#ifdef HAVE_STARTUP_NOTIFICATION
+#if defined (HAVE_STARTUP_NOTIFICATION) && defined (HAVE_X11)
 static void
 mx_application_window_map_cb (ClutterActor  *actor,
                               GParamSpec    *pspec,
@@ -666,7 +634,7 @@ mx_application_new (gint                *argc,
   MxApplication *app;
 
   /* initialise clutter and the type system */
-  clutter_init (argc, argv);
+  clutter_init_with_args (argc, argv, name, NULL, NULL, NULL);
 
   mx_set_locale ();
 
@@ -725,191 +693,6 @@ mx_application_quit (MxApplication *application)
   clutter_main_quit ();
 }
 
-static void
-mx_application_refresh_wm_props (MxApplication *self)
-{
-  unsigned long n_items, bytes_left;
-  unsigned char *return_string;
-  int return_format;
-  Atom return_type;
-  Display *dpy;
-  GList *w;
-
-  MxApplicationPrivate *priv = self->priv;
-
-  if (!priv->atoms[MX_ATOM_MOBLIN] ||
-      !priv->wm_window)
-    return;
-
-  dpy = clutter_x11_get_default_display ();
-
-  /* Get the Moblin WM properties string */
-  clutter_x11_trap_x_errors ();
-  return_string = NULL;
-  XGetWindowProperty (dpy, *priv->wm_window,
-                      priv->atoms[MX_ATOM_MOBLIN],
-                      0, 8192, False, XA_STRING,
-                      &return_type, &return_format,
-                      &n_items, &bytes_left, &return_string);
-  clutter_x11_untrap_x_errors ();
-
-  /* The _MOBLIN properties string is a list of 'key=value' pairs,
-   * delimited by colons.
-   */
-  if (return_string)
-    {
-      gchar *prop = g_strdelimit ((gchar *)return_string, ":", '\0');
-
-      priv->small_screen = FALSE;
-      while (*prop)
-        {
-          gchar *key = g_strdelimit (prop, "=", '\0');
-          gchar *value = key + strlen (key) + 1;
-
-          /* Check for session-type=small-screen - the only property
-           * we support, currently.
-           */
-          if (g_str_equal (key, "session-type") &&
-              g_str_equal (value, "small-screen"))
-            priv->small_screen = TRUE;
-
-          prop = value + strlen (value) + 1;
-        }
-
-      XFree (return_string);
-    }
-
-  for (w = priv->windows; w; w = w->next)
-    if (MX_IS_WINDOW (w->data))
-      mx_window_set_small_screen (MX_WINDOW (w->data), priv->small_screen);
-}
-
-static void
-mx_application_refresh_wm_window (MxApplication *self)
-{
-  unsigned long n_items, bytes_left;
-  int return_format;
-  Atom return_type;
-  Window root_win;
-  Display *dpy;
-
-  MxApplicationPrivate *priv = self->priv;
-
-  if (!priv->atoms[MX_ATOM_NET_SUPPORTING_WM_CHECK])
-    return;
-
-  root_win = clutter_x11_get_root_window ();
-  dpy = clutter_x11_get_default_display ();
-
-  if (priv->wm_window)
-    {
-      XFree (priv->wm_window);
-      priv->wm_window = NULL;
-    }
-
-  /* Get the WM window */
-  clutter_x11_trap_x_errors ();
-  XGetWindowProperty (dpy, root_win,
-                      priv->atoms[MX_ATOM_NET_SUPPORTING_WM_CHECK],
-                      0, 1, False, XA_WINDOW,
-                      &return_type, &return_format,
-                      &n_items, &bytes_left,
-                      (unsigned char **)(&priv->wm_window));
-  clutter_x11_untrap_x_errors ();
-
-  if (priv->wm_window && (*priv->wm_window == None))
-    {
-      XFree (priv->wm_window);
-      priv->wm_window = NULL;
-    }
-
-  if (priv->wm_window)
-    {
-      XWindowAttributes attr;
-
-      /* Add property-change notification */
-      if (XGetWindowAttributes (dpy, *priv->wm_window, &attr))
-        XSelectInput (dpy, *priv->wm_window,
-                      attr.your_event_mask | PropertyChangeMask);
-
-      /* Refresh the WM properties */
-      mx_application_refresh_wm_props (self);
-    }
-}
-
-static ClutterX11FilterReturn
-mx_application_x11_filter_func (XEvent       *xevent,
-                                ClutterEvent *cevent,
-                                gpointer      data)
-{
-  Window root_win;
-  MxApplication *self;
-  MxApplicationPrivate *priv;
-
-  self = data;
-  priv = self->priv;
-  root_win = clutter_x11_get_root_window ();
-
-  switch (xevent->type)
-    {
-    case PropertyNotify:
-      if (xevent->xproperty.window == root_win)
-        {
-          if (xevent->xproperty.atom ==
-              priv->atoms[MX_ATOM_NET_SUPPORTING_WM_CHECK])
-            mx_application_refresh_wm_window (self);
-        }
-      else if (priv->wm_window &&
-               (xevent->xproperty.window == *priv->wm_window))
-        {
-          if (xevent->xproperty.atom == priv->atoms[MX_ATOM_MOBLIN])
-            mx_application_refresh_wm_props (self);
-        }
-      break;
-
-    case DestroyNotify:
-      if (priv->wm_window &&
-          (xevent->xdestroywindow.window == *priv->wm_window))
-        {
-          XFree (priv->wm_window);
-          priv->wm_window = NULL;
-        }
-      break;
-    }
-
-  return CLUTTER_X11_FILTER_CONTINUE;
-}
-
-static void
-mx_application_init_wm (MxApplication *self)
-{
-  Display *dpy;
-  Window root_win;
-  XWindowAttributes attr;
-
-  MxApplicationPrivate *priv = self->priv;
-
-  dpy = clutter_x11_get_default_display ();
-
-  /* Create the relevant Atoms for reading WM properties */
-  XInternAtoms (dpy,
-                mx_application_atoms,
-                G_N_ELEMENTS (mx_application_atoms),
-                False,
-                priv->atoms);
-
-  /* Add X property change notifications to the event mask */
-  root_win = clutter_x11_get_root_window ();
-  if (XGetWindowAttributes (dpy, root_win, &attr))
-    XSelectInput (dpy, root_win, attr.your_event_mask | PropertyChangeMask);
-
-  /* Add our own events filter for property change/destroy notification */
-  clutter_x11_add_filter (mx_application_x11_filter_func, self);
-
-  /* Read the current properties */
-  mx_application_refresh_wm_window (self);
-}
-
 /**
  * mx_application_get_flags:
  * @application: an #MxApplication
@@ -952,11 +735,12 @@ mx_application_add_window (MxApplication *application,
                     G_CALLBACK (mx_application_window_destroy_cb), application);
 
   /* Use the first window of the application for startup notification and
-   * initialising X11 property reading.
+   * mirroring small-screen mode.
    */
   if (first_window)
     {
-#ifdef HAVE_STARTUP_NOTIFICATION
+      gboolean small_screen;
+#if defined (HAVE_X11) && defined (HAVE_STARTUP_NOTIFICATION)
       ClutterStage *stage;
       SnDisplay *display;
       Display *xdisplay;
@@ -965,9 +749,7 @@ mx_application_add_window (MxApplication *application,
 
       first_window = FALSE;
 
-      mx_application_init_wm (application);
-
-#ifdef HAVE_STARTUP_NOTIFICATION
+#if defined (HAVE_X11) && defined (HAVE_STARTUP_NOTIFICATION)
       xdisplay = clutter_x11_get_default_display ();
       screen = clutter_x11_get_default_screen ();
       stage = mx_window_get_clutter_stage (window);
@@ -991,9 +773,17 @@ mx_application_add_window (MxApplication *application,
                               application);
         }
 #endif
+
+      g_object_get (G_OBJECT (mx_settings_get_default ()),
+                    "small-screen", &small_screen, NULL);
+      mx_window_set_small_screen (window, small_screen);
     }
   else
-    mx_window_set_small_screen (window, priv->small_screen);
+    {
+      /* FIXME: Other windows in the application should probably be marked
+       *        as tool windows, or something special/clever like that.
+       */
+    }
 }
 
 /**
@@ -1008,24 +798,24 @@ void
 mx_application_remove_window (MxApplication *application,
                               MxWindow      *window)
 {
-  GList *link;
+  GList *list;
 
   g_return_if_fail (MX_IS_APPLICATION (application));
   g_return_if_fail (MX_IS_WINDOW (window));
 
-  link = g_list_find (application->priv->windows, window);
+  list = g_list_find (application->priv->windows, window);
 
-  if (!link)
+  if (!list)
     {
       g_warning ("Could not remove window from application, the window was not"
                  " found in the application's window list");
       return;
     }
 
-  g_object_unref (G_OBJECT (link->data));
+  g_object_unref (G_OBJECT (list->data));
 
   application->priv->windows = g_list_delete_link (application->priv->windows,
-                                                   link);
+                                                   list);
 }
 
 
