@@ -64,6 +64,7 @@
 static void mx_box_container_iface_init (ClutterContainerIface *iface);
 static void mx_box_scrollable_interface_init (MxScrollableIface *iface);
 static void mx_box_focusable_iface_init (MxFocusableIface *iface);
+static void mx_box_stylable_iface_init (MxStylableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxBoxLayout, mx_box_layout, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
@@ -71,7 +72,9 @@ G_DEFINE_TYPE_WITH_CODE (MxBoxLayout, mx_box_layout, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (MX_TYPE_SCROLLABLE,
                                                 mx_box_scrollable_interface_init)
                          G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
-                                                mx_box_focusable_iface_init));
+                                                mx_box_focusable_iface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
+                                                mx_box_stylable_iface_init));
 
 #define BOX_LAYOUT_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MX_TYPE_BOX_LAYOUT, MxBoxLayoutPrivate))
@@ -85,13 +88,17 @@ enum {
   PROP_HADJUST,
   PROP_VADJUST,
 
-  PROP_ENABLE_ANIMATIONS
+  PROP_ENABLE_ANIMATIONS,
+  PROP_SCROLL_TO_FOCUSED
 };
 
 struct _MxBoxLayoutPrivate
 {
   GList        *children;
 
+  guint         ignore_css_spacing : 1; /* Should we ignore spacing from
+                                           the CSS because the application
+                                           set it via set_spacing */
   guint         spacing;
 
   MxAdjustment *hadjustment;
@@ -102,6 +109,7 @@ struct _MxBoxLayoutPrivate
   ClutterAlpha    *alpha;
   guint            is_animating : 1;
   guint            enable_animations : 1;
+  guint            scroll_to_focused : 1;
 
   MxOrientation orientation;
 
@@ -337,8 +345,6 @@ mx_box_container_remove_actor (ClutterContainer *container,
 
   g_object_ref (actor);
 
-  priv = MX_BOX_LAYOUT (container)->priv;
-
   if ((ClutterActor *)priv->last_focus == actor)
     priv->last_focus = NULL;
 
@@ -463,6 +469,9 @@ update_adjustments (MxBoxLayout *self,
   ClutterActorBox box = { 0, };
   clutter_actor_get_allocation_box (CLUTTER_ACTOR (focusable), &box);
 
+  if (!priv->scroll_to_focused)
+    return;
+
   if (priv->vadjustment)
     {
       mx_adjustment_get_values (priv->vadjustment,
@@ -503,6 +512,7 @@ mx_box_layout_move_focus (MxFocusable      *focusable,
 {
   MxBoxLayoutPrivate *priv = MX_BOX_LAYOUT (focusable)->priv;
   GList *l, *childlink;
+  MxFocusHint hint;
 
   /* find the current focus */
   childlink = g_list_find (priv->children, from);
@@ -510,6 +520,8 @@ mx_box_layout_move_focus (MxFocusable      *focusable,
   if (!childlink)
     return NULL;
 
+  hint = mx_focus_hint_from_direction (direction);
+
   /* convert left/right and up/down into next/previous */
   if (priv->orientation == MX_ORIENTATION_HORIZONTAL)
     {
@@ -536,7 +548,7 @@ mx_box_layout_move_focus (MxFocusable      *focusable,
               MxFocusable *focused, *child;
 
               child = MX_FOCUSABLE (l->data);
-              focused = mx_focusable_accept_focus (child, MX_FOCUS_HINT_FIRST);
+              focused = mx_focusable_accept_focus (child, hint);
 
               if (focused)
                 {
@@ -546,9 +558,6 @@ mx_box_layout_move_focus (MxFocusable      *focusable,
                 }
             }
         }
-
-      /* no next widgets to focus */
-      return NULL;
     }
   else if (direction == MX_FOCUS_DIRECTION_PREVIOUS)
     {
@@ -559,7 +568,7 @@ mx_box_layout_move_focus (MxFocusable      *focusable,
               MxFocusable *focused, *child;
 
               child = MX_FOCUSABLE (l->data);
-              focused = mx_focusable_accept_focus (child, MX_FOCUS_HINT_LAST);
+              focused = mx_focusable_accept_focus (child, hint);
 
               if (focused)
                 {
@@ -579,17 +588,36 @@ mx_box_layout_accept_focus (MxFocusable *focusable, MxFocusHint hint)
 {
   MxBoxLayoutPrivate *priv = MX_BOX_LAYOUT (focusable)->priv;
   MxFocusable *return_focusable;
+  MxFocusHint modified_hint;
   GList* list, *l;
 
   return_focusable = NULL;
 
-  /* find the first/last focusable widget */
-  switch (hint)
+  /* Transform the hint based on our orientation */
+  modified_hint = hint;
+  if (priv->orientation == MX_ORIENTATION_HORIZONTAL)
+    {
+      if (hint == MX_FOCUS_HINT_FROM_RIGHT)
+        modified_hint = MX_FOCUS_HINT_LAST;
+      else if (hint == MX_FOCUS_HINT_FROM_LEFT)
+        modified_hint = MX_FOCUS_HINT_FIRST;
+    }
+  else
+    {
+      if (hint == MX_FOCUS_HINT_FROM_BELOW)
+        modified_hint = MX_FOCUS_HINT_LAST;
+      else if (hint == MX_FOCUS_HINT_FROM_ABOVE)
+        modified_hint = MX_FOCUS_HINT_FIRST;
+    }
+
+  /* find the first/last/prior focusable widget */
+  switch (modified_hint)
     {
     case MX_FOCUS_HINT_LAST:
       list = g_list_reverse (g_list_copy (priv->children));
       break;
 
+    default:
     case MX_FOCUS_HINT_PRIOR:
       if (priv->last_focus)
         {
@@ -599,7 +627,6 @@ mx_box_layout_accept_focus (MxFocusable *focusable, MxFocusHint hint)
         }
       /* This intentionally runs into the next case */
 
-    default:
     case MX_FOCUS_HINT_FIRST:
       list = g_list_copy (priv->children);
       break;
@@ -632,6 +659,26 @@ mx_box_focusable_iface_init (MxFocusableIface *iface)
   iface->accept_focus = mx_box_layout_accept_focus;
 }
 
+/* Stylable inplementation */
+static void
+mx_box_stylable_iface_init (MxStylableIface *iface)
+{
+  static gboolean is_initialized = FALSE;
+
+  if (G_UNLIKELY (!is_initialized))
+    {
+      GParamSpec *pspec;
+
+      is_initialized = TRUE;
+
+      pspec = g_param_spec_uint ("x-mx-spacing",
+                                 "Spacing",
+                                 "The size of the spacing",
+                                 0, G_MAXUINT, 0,
+                                 MX_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_BOX_LAYOUT, pspec);
+    }
+}
 
 static void
 mx_box_layout_get_property (GObject    *object,
@@ -656,6 +703,10 @@ mx_box_layout_get_property (GObject    *object,
       g_value_set_boolean (value, priv->enable_animations);
       break;
 
+    case PROP_SCROLL_TO_FOCUSED:
+      g_value_set_boolean (value, priv->scroll_to_focused);
+      break;
+
     case PROP_HADJUST:
       scrollable_get_adjustments (MX_SCROLLABLE (object), &adjustment, NULL);
       g_value_set_object (value, adjustment);
@@ -705,6 +756,10 @@ mx_box_layout_set_property (GObject      *object,
                                   g_value_get_object (value));
       break;
 
+    case PROP_SCROLL_TO_FOCUSED:
+      mx_box_layout_set_scroll_to_focused (box, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -1179,7 +1234,6 @@ mx_box_layout_allocate (ClutterActor          *actor,
           ClutterActorBox *start, *end, now;
           gdouble alpha;
 
-
           ClutterActorBox *copy = g_new (ClutterActorBox, 1);
 
           *copy = child_box;
@@ -1193,7 +1247,7 @@ mx_box_layout_allocate (ClutterActor          *actor,
               /* don't know where this actor was from (possibly recently
                * added), so just allocate the end co-ordinates */
               clutter_actor_allocate (child, end, flags);
-              continue;
+              goto next;
             }
 
           now.x1 = (int) (start->x1 + (end->x1 - start->x1) * alpha);
@@ -1222,6 +1276,7 @@ mx_box_layout_allocate (ClutterActor          *actor,
           clutter_actor_allocate (child, copy, flags);
         }
 
+next:
       if (priv->orientation == MX_ORIENTATION_VERTICAL)
         position += (old_child_box.y2 - old_child_box.y1) + priv->spacing;
       else
@@ -1396,6 +1451,13 @@ mx_box_layout_class_init (MxBoxLayoutClass *klass)
                                 MX_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_ENABLE_ANIMATIONS, pspec);
 
+  pspec = g_param_spec_boolean ("scroll-to-focused",
+                                "Scroll to focused",
+                                "Automatically scroll to the focused actor",
+                                TRUE,
+                                MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_SCROLL_TO_FOCUSED, pspec);
+
   /* MxScrollable properties */
   g_object_class_override_property (object_class,
                                     PROP_HADJUST,
@@ -1413,6 +1475,27 @@ mx_box_layout_free_allocation (ClutterActorBox *box)
   g_boxed_free (CLUTTER_TYPE_ACTOR_BOX, box);
 }
 
+static void
+mx_box_layout_style_changed (MxWidget *widget,
+                             gpointer  userdata)
+{
+  MxBoxLayout *layout = MX_BOX_LAYOUT (widget);
+  MxBoxLayoutPrivate *priv = layout->priv;
+  guint spacing;
+
+  mx_stylable_get (MX_STYLABLE (widget),
+                   "x-mx-spacing", &spacing,
+                   NULL);
+
+  if (!priv->ignore_css_spacing && (priv->spacing != spacing))
+    {
+      priv->spacing = spacing;
+      clutter_actor_queue_relayout (CLUTTER_ACTOR (widget));
+    }
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (widget));
+}
+
 static void
 mx_box_layout_init (MxBoxLayout *self)
 {
@@ -1425,6 +1508,10 @@ mx_box_layout_init (MxBoxLayout *self)
                                                          (GDestroyNotify)
                                                          mx_box_layout_free_allocation);
 
+  g_signal_connect (self, "style-changed",
+                    G_CALLBACK (mx_box_layout_style_changed), NULL);
+
+  self->priv->scroll_to_focused = TRUE;
 }
 
 /**
@@ -1501,6 +1588,7 @@ mx_box_layout_set_spacing (MxBoxLayout *box,
   if (priv->spacing != spacing)
     {
       priv->spacing = spacing;
+      priv->ignore_css_spacing = TRUE;
 
       clutter_actor_queue_relayout (CLUTTER_ACTOR (box));
 
@@ -1719,3 +1807,42 @@ mx_box_layout_add_actor_with_properties (MxBoxLayout  *box,
   mx_box_layout_set_property_valist (box, actor, first_property, var_args);
   va_end (var_args);
 }
+
+/**
+ * mx_box_layout_set_scroll_to_focused:
+ * @box: A #MxBoxLayout
+ * @scroll_to_focused: #TRUE to enable automatically scrolling to the
+ *   focused actor
+ *
+ * Enables or disables automatic scrolling to the focused actor.
+ */
+void
+mx_box_layout_set_scroll_to_focused (MxBoxLayout *box,
+                                     gboolean     scroll_to_focused)
+{
+  MxBoxLayoutPrivate *priv;
+
+  g_return_if_fail (MX_IS_BOX_LAYOUT (box));
+
+  priv = box->priv;
+  if (priv->scroll_to_focused != scroll_to_focused)
+    {
+      priv->scroll_to_focused = scroll_to_focused;
+      g_object_notify (G_OBJECT (box), "scroll-to-focused");
+    }
+}
+
+/**
+ * mx_box_layout_get_scroll_to_focused:
+ * @box: A #MxBoxLayout
+ *
+ * Get the value of the #MxBoxLayout:scroll-to-focused property.
+ *
+ * Returns: #TRUE if automatically scrolling to the focused actor is enabled
+ */
+gboolean
+mx_box_layout_get_scroll_to_focused (MxBoxLayout *box)
+{
+  g_return_val_if_fail (MX_IS_BOX_LAYOUT (box), FALSE);
+  return box->priv->scroll_to_focused;
+}
