@@ -460,13 +460,16 @@ mx_combo_box_allocate (ClutterActor          *actor,
   stage = clutter_actor_get_stage (actor);
   if (stage != NULL)
     {
-      gfloat trans_x, trans_y, stage_h, combo_h = box->y2 - box->y1;
+      ClutterVertex point = { 0, };
+      gfloat stage_w, stage_h, combo_h = box->y2 - box->y1;
 
-      stage_h = clutter_actor_get_height (stage);
-      clutter_actor_get_transformed_position (actor, &trans_x, &trans_y);
+      clutter_actor_get_size (stage, &stage_w, &stage_h);
+      point.y = combo_h + nat_menu_h;
+      clutter_actor_apply_transform_to_point (actor, &point, &point);
 
-      if ( ((trans_y + nat_menu_h + combo_h) > stage_h) &&
-           (stage_h - combo_h) > 0 )
+      /* If the menu would appear off the stage, flip it around. */
+      if ((point.x < 0) || (point.x >= stage_w) ||
+          (point.y < 0) || (point.y >= stage_h))
         {
           childbox.y1 = -nat_menu_h;
         }
@@ -544,12 +547,12 @@ mx_combo_box_key_press_event (ClutterActor    *actor,
 
   switch (event->keyval)
     {
-    case CLUTTER_Down:
+    case CLUTTER_KEY_Down:
       mx_combo_box_set_index (MX_COMBO_BOX (actor),
                               CLAMP (priv->index + 1, 0, n_items -1));
       return TRUE;
 
-    case CLUTTER_Up:
+    case CLUTTER_KEY_Up:
       mx_combo_box_set_index (MX_COMBO_BOX (actor),
                               CLAMP (priv->index - 1, 0, n_items -1));
       return TRUE;
@@ -559,6 +562,15 @@ mx_combo_box_key_press_event (ClutterActor    *actor,
     }
 }
 
+static void
+mx_combo_box_apply_style (MxWidget *widget,
+                          MxStyle  *style)
+{
+  MxComboBoxPrivate *priv = MX_COMBO_BOX (widget)->priv;
+
+  if (priv->icon != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->icon), style);
+}
 
 static void
 mx_combo_box_style_changed (MxComboBox *combo, MxStyleChangedFlags flags)
@@ -609,6 +621,7 @@ mx_combo_box_class_init (MxComboBoxClass *klass)
   GParamSpec *pspec;
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  MxWidgetClass *widget_class = MX_WIDGET_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (MxComboBoxPrivate));
 
@@ -628,6 +641,8 @@ mx_combo_box_class_init (MxComboBoxClass *klass)
   actor_class->button_press_event = mx_combo_box_button_press_event;
   actor_class->key_press_event = mx_combo_box_key_press_event;
 
+  widget_class->apply_style = mx_combo_box_apply_style;
+
   pspec = g_param_spec_string ("active-text",
                                "Active Text",
                                "Text currently displayed in the combo box"
@@ -830,9 +845,14 @@ mx_combo_box_remove_text (MxComboBox *box,
   GSList *item;
 
   g_return_if_fail (MX_IS_COMBO_BOX (box));
+  g_return_if_fail (position >= 0);
 
   /* find the item, free the string and remove it from the list */
   item = g_slist_nth (box->priv->actions, position);
+
+  if (!item)
+    return;
+
   g_object_unref (G_OBJECT (item->data));
   box->priv->actions = g_slist_delete_link (box->priv->actions, item);
   mx_combo_box_update_menu (box);
