@@ -25,8 +25,8 @@
  * SECTION:mx-entry
  * @short_description: Widget for displaying text
  *
- * #MxEntry is a simple widget for displaying text. It derives from
- * #MxWidget to add extra style and placement functionality over
+ * #MxEntry is a widget for displaying and editing a single line of text.
+ * It derives from #MxWidget to add extra style and placement functionality over
  * #ClutterText. The internal #ClutterText is publicly accessibly to allow
  * applications to set further properties.
  *
@@ -65,15 +65,17 @@
 #include "mx-marshal.h"
 #include "mx-clipboard.h"
 #include "mx-focusable.h"
+#include "mx-private.h"
 
+#ifdef HAVE_X11
 /* for pointer cursor support */
 #include <clutter/x11/clutter-x11.h>
 #include <X11/Xlib.h>
 #include <X11/cursorfont.h>
+#endif
 
 #define HAS_FOCUS(actor) (clutter_actor_get_stage (actor) && clutter_stage_get_key_focus ((ClutterStage *) clutter_actor_get_stage (actor)) == actor)
 
-
 /* properties */
 enum
 {
@@ -114,6 +116,8 @@ struct _MxEntryPrivate
   gboolean  pause_undo;
   GQueue   *undo_history;
   gulong    undo_timeout_source;
+
+  gboolean scrolling;
 };
 
 static guint entry_signals[LAST_SIGNAL] = { 0, };
@@ -273,27 +277,14 @@ static void
 mx_entry_style_changed (MxWidget *self)
 {
   MxEntryPrivate *priv = MX_ENTRY_PRIV (self);
-  ClutterColor *color = NULL;
   ClutterColor *caret_color = NULL;
   ClutterColor *selection_background_color = NULL;
-  gchar *font_name;
-  gchar *font_string;
-  gint font_size;
 
   mx_stylable_get (MX_STYLABLE (self),
-                   "color", &color,
                    "caret-color", &caret_color,
                    "selection-background-color", &selection_background_color,
-                   "font-family", &font_name,
-                   "font-size", &font_size,
                    NULL);
 
-  if (color)
-    {
-      clutter_text_set_color (CLUTTER_TEXT (priv->entry), color);
-      clutter_color_free (color);
-    }
-
   if (caret_color)
     {
       clutter_text_set_cursor_color (CLUTTER_TEXT (priv->entry), caret_color);
@@ -307,24 +298,8 @@ mx_entry_style_changed (MxWidget *self)
       clutter_color_free (selection_background_color);
     }
 
-  if (font_name || font_size)
-    {
-      if (font_name && font_size)
-        {
-          font_string = g_strdup_printf ("%s %dpx", font_name, font_size);
-          g_free (font_name);
-        }
-      else
-        {
-          if (font_size)
-            font_string = g_strdup_printf ("%dpx", font_size);
-          else
-            font_string = font_name;
-        }
-
-      clutter_text_set_font_name (CLUTTER_TEXT (priv->entry), font_string);
-      g_free (font_string);
-    }
+  mx_stylable_apply_clutter_text_attributes (MX_STYLABLE (self),
+                                             CLUTTER_TEXT (priv->entry));
 }
 
 static void
@@ -441,7 +416,7 @@ mx_entry_allocate (ClutterActor          *actor,
   ClutterActorBox child_box, icon_box;
   MxPadding padding;
   gfloat icon_w, icon_h;
-  gfloat entry_h, min_h, pref_h, avail_h;
+  gfloat entry_h, min_h, pref_h, avail_h, pref_w;
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
 
@@ -500,6 +475,10 @@ mx_entry_allocate (ClutterActor          *actor,
 
   entry_h = CLAMP (pref_h, min_h, avail_h);
 
+  clutter_actor_get_preferred_width (priv->entry, entry_h, NULL, &pref_w);
+
+  priv->scrolling = (pref_w > (child_box.x2 - child_box.x1));
+
   child_box.y1 = (int)(padding.top + avail_h / 2 - entry_h / 2);
   child_box.y2 = child_box.y1 + entry_h;
 
@@ -534,6 +513,9 @@ clutter_text_focus_out_cb (ClutterText  *text,
 {
   MxEntryPrivate *priv = MX_ENTRY_PRIV (actor);
 
+  /* move the cursor back to the beginning of the entry */
+  clutter_text_set_cursor_position (CLUTTER_TEXT (priv->entry), 0);
+
   /* add a hint if the entry is empty */
   if (priv->hint && !strcmp (clutter_text_get_text (text), ""))
     {
@@ -559,6 +541,7 @@ mx_entry_paint (ClutterActor *actor)
 {
   MxEntryPrivate *priv = MX_ENTRY_PRIV (actor);
   ClutterActorClass *parent_class;
+  ClutterStage *stage;
 
   parent_class = CLUTTER_ACTOR_CLASS (mx_entry_parent_class);
   parent_class->paint (actor);
@@ -570,6 +553,45 @@ mx_entry_paint (ClutterActor *actor)
 
   if (priv->secondary_icon)
     clutter_actor_paint (priv->secondary_icon);
+
+
+  /* draw a shadow if the entry is not focused and the text is scrolling */
+  stage = (ClutterStage *) clutter_actor_get_stage (priv->entry);
+
+  if (clutter_stage_get_key_focus (stage) != priv->entry && priv->scrolling)
+    {
+      ClutterGeometry geo;
+      CoglTextureVertex top[4] = { { 0,}, };
+      ClutterColor *color;
+      guint8 r, g, b;
+
+
+      mx_stylable_get (MX_STYLABLE (actor), "background-color", &color, NULL);
+
+      r = color->red;
+      g = color->green;
+      b = color->blue;
+      clutter_color_free (color);
+
+      cogl_set_source_color4ub (0, 0, 0, 0);
+
+      clutter_actor_get_allocation_geometry (priv->entry, &geo);
+
+      top[0].x = geo.x + geo.width;
+      top[0].y = geo.y + geo.height;
+      top[1].x = geo.x + geo.width;
+      top[1].y = geo.y;
+      top[2].x = geo.x + geo.width - 30;
+      top[2].y = geo.y;
+      top[3].x = geo.x + geo.width - 30;
+      top[3].y = geo.y + geo.height;
+
+      cogl_color_set_from_4ub (&top[0].color, r, g, b, 0xff);
+      cogl_color_set_from_4ub (&top[1].color, r, g, b, 0xff);
+      cogl_color_set_from_4ub (&top[2].color, 0, 0, 0, 0);
+      cogl_color_set_from_4ub (&top[3].color, 0, 0, 0, 0);
+      cogl_polygon (top, 4, TRUE);
+    }
 }
 
 static void
@@ -655,7 +677,7 @@ mx_entry_key_press_event (ClutterActor    *actor,
 
   /* paste */
   if ((event->modifier_state & CLUTTER_CONTROL_MASK)
-      && event->keyval == CLUTTER_v)
+      && event->keyval == CLUTTER_KEY_v)
     {
       MxClipboard *clipboard;
 
@@ -668,7 +690,7 @@ mx_entry_key_press_event (ClutterActor    *actor,
 
   /* copy */
   if ((event->modifier_state & CLUTTER_CONTROL_MASK)
-      && event->keyval == CLUTTER_c)
+      && event->keyval == CLUTTER_KEY_c)
     {
       MxClipboard *clipboard;
       gchar *text;
@@ -686,7 +708,7 @@ mx_entry_key_press_event (ClutterActor    *actor,
 
   /* cut */
   if ((event->modifier_state & CLUTTER_CONTROL_MASK)
-      && event->keyval == CLUTTER_x)
+      && event->keyval == CLUTTER_KEY_x)
     {
       MxClipboard *clipboard;
       gchar *text;
@@ -708,7 +730,7 @@ mx_entry_key_press_event (ClutterActor    *actor,
 
   /* undo */
   if ((event->modifier_state & CLUTTER_CONTROL_MASK)
-      && event->keyval == CLUTTER_z)
+      && event->keyval == CLUTTER_KEY_z)
     {
       gchar *str;
 
@@ -749,6 +771,7 @@ mx_entry_swallow_crossing_event (ClutterActor         *actor,
                                  ClutterCrossingEvent *event)
 {
 
+#ifdef HAVE_X11
   if (event->source == MX_ENTRY (actor)->priv->entry
       && event->related != NULL)
     {
@@ -769,6 +792,7 @@ mx_entry_swallow_crossing_event (ClutterActor         *actor,
       else
         XUndefineCursor (dpy, wid);
     }
+#endif
 
   /* swallow enter and leave events, since the pseudo-class must not be set to
    * 'hover' because it would loose the 'focus' state.
@@ -931,7 +955,7 @@ mx_entry_store_undo_on_keypress (ClutterText     *text,
                                  ClutterKeyEvent *event,
                                  MxEntry         *entry)
 {
-  if (event->keyval == CLUTTER_space)
+  if (event->keyval == CLUTTER_KEY_space)
     {
       /* store the text immediately */
       if (entry->priv->undo_timeout_source)
