@@ -515,6 +515,16 @@ mx_expander_unmap (ClutterActor *actor)
   clutter_actor_unmap (priv->arrow);
 }
 
+static void
+mx_expander_apply_style (MxWidget *widget,
+                         MxStyle  *style)
+{
+  MxExpanderPrivate *priv = MX_EXPANDER (widget)->priv;
+
+  if (priv->arrow != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->arrow), style);
+}
+
 static void
 mx_expander_style_changed (MxStylable *stylable)
 {
@@ -579,6 +589,7 @@ mx_expander_class_init (MxExpanderClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  MxWidgetClass *widget_class = MX_WIDGET_CLASS (klass);
   GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (MxExpanderPrivate));
@@ -596,6 +607,8 @@ mx_expander_class_init (MxExpanderClass *klass)
   actor_class->map = mx_expander_map;
   actor_class->unmap = mx_expander_unmap;
 
+  widget_class->apply_style = mx_expander_apply_style;
+
   pspec = g_param_spec_boolean ("expanded",
                                 "Expanded",
                                 "Indicates that the expander is open or closed",
@@ -608,7 +621,8 @@ mx_expander_class_init (MxExpanderClass *klass)
   pspec = g_param_spec_string ("label",
                                "Label",
                                "Expander title label.",
-                               NULL, G_PARAM_READWRITE);
+                               NULL,
+                               G_PARAM_READWRITE | MX_PARAM_TRANSLATEABLE);
   g_object_class_install_property (object_class,
                                    PROP_LABEL,
                                    pspec);
