@@ -225,26 +225,27 @@ mx_focus_manager_event_cb (ClutterStage   *stage,
   old_focus = priv->focused;
   switch (event->key.keyval)
     {
-    case CLUTTER_Tab:
+    case CLUTTER_KEY_Tab:
+    case CLUTTER_KEY_ISO_Left_Tab:
       if (event->key.modifier_state & CLUTTER_SHIFT_MASK)
         mx_focus_manager_move_focus (manager, MX_FOCUS_DIRECTION_PREVIOUS);
       else
         mx_focus_manager_move_focus (manager, MX_FOCUS_DIRECTION_NEXT);
       break;
 
-    case CLUTTER_Right:
+    case CLUTTER_KEY_Right:
       mx_focus_manager_move_focus (manager, MX_FOCUS_DIRECTION_RIGHT);
       break;
 
-    case CLUTTER_Left:
+    case CLUTTER_KEY_Left:
       mx_focus_manager_move_focus (manager, MX_FOCUS_DIRECTION_LEFT);
       break;
 
-    case CLUTTER_Up:
+    case CLUTTER_KEY_Up:
       mx_focus_manager_move_focus (manager, MX_FOCUS_DIRECTION_UP);
       break;
 
-    case CLUTTER_Down:
+    case CLUTTER_KEY_Down:
       mx_focus_manager_move_focus (manager, MX_FOCUS_DIRECTION_DOWN);
       break;
 
@@ -258,6 +259,15 @@ mx_focus_manager_event_cb (ClutterStage   *stage,
     return TRUE;
 }
 
+/**
+ * mx_focus_manager_get_for_stage:
+ * @stage: A #ClutterStage
+ *
+ * Get the MxFocusManager associated with a stage, or create one if none exist
+ * for the specified stage.
+ *
+ * Returns: (transfer none): An #MxFocusManager
+ */
 MxFocusManager *
 mx_focus_manager_get_for_stage (ClutterStage *stage)
 {
@@ -289,6 +299,14 @@ mx_focus_manager_get_for_stage (ClutterStage *stage)
   return manager;
 }
 
+/**
+ * mx_focus_manager_get_stage:
+ * @manager: A #MxFocusManager
+ *
+ * Get the stage the MxFocusManager is associated with
+ *
+ * Returns: (transfer none): A #ClutterStage
+ */
 ClutterStage*
 mx_focus_manager_get_stage (MxFocusManager *manager)
 {
@@ -297,6 +315,14 @@ mx_focus_manager_get_stage (MxFocusManager *manager)
   return CLUTTER_STAGE (manager->priv->stage);
 }
 
+/**
+ * mx_focus_manager_get_focused:
+ * @manager: A #MxFocusManager
+ *
+ * Get the currently focused #MxFocusable
+ *
+ * Returns: (transfer none): MxFocusable
+ */
 MxFocusable*
 mx_focus_manager_get_focused (MxFocusManager *manager)
 {
@@ -310,6 +336,9 @@ mx_focus_manager_get_focused (MxFocusManager *manager)
  * @manager: the focus manager
  * @focusable: the object to set focus on
  *
+ * Sets the currently focused actor, with an #MxFocusHint of
+ * %MX_FOCUS_HINT_PRIOR.
+ *
  * Note: the final focused object may not be the same as @focusable if
  * @focusable does not accept focus directly.
  */
@@ -332,7 +361,7 @@ mx_focus_manager_push_focus (MxFocusManager *manager,
           mx_focusable_move_focus (priv->focused, MX_FOCUS_DIRECTION_OUT, priv->focused);
         }
 
-      priv->focused = mx_focusable_accept_focus (focusable, MX_FOCUS_HINT_FIRST);
+      priv->focused = mx_focusable_accept_focus (focusable, MX_FOCUS_HINT_PRIOR);
 
       g_object_notify (G_OBJECT (manager), "focused");
     }
@@ -346,8 +375,8 @@ mx_focus_manager_push_focus (MxFocusManager *manager,
  * Moves the current focus in the given direction.
  */
 void
-mx_focus_manager_move_focus  (MxFocusManager   *manager,
-                              MxFocusDirection  direction)
+mx_focus_manager_move_focus (MxFocusManager   *manager,
+                             MxFocusDirection  direction)
 {
   MxFocusable *old_focus;
   MxFocusManagerPrivate *priv;
@@ -366,7 +395,7 @@ mx_focus_manager_move_focus  (MxFocusManager   *manager,
   if (!priv->focused)
     {
       /* If we're going next or previous, we wrap around, otherwise
-       * we lose focus.
+       * re-focus the last actor.
        */
       switch (direction)
         {
@@ -389,6 +418,13 @@ mx_focus_manager_move_focus  (MxFocusManager   *manager,
           break;
 
         default:
+          /* re-focus the original */
+          if (old_focus && (direction != MX_FOCUS_DIRECTION_OUT))
+            priv->focused = mx_focusable_accept_focus (old_focus, 0);
+          else
+            mx_focus_manager_ensure_focused (manager,
+                                             CLUTTER_STAGE (priv->stage),
+                                             MX_FOCUS_HINT_FIRST);
           break;
         }
     }
