@@ -27,6 +27,7 @@
 #include "mx-marshal.h"
 #include "mx-texture-cache.h"
 #include "mx-private.h"
+#include "mx-settings.h"
 
 G_DEFINE_TYPE (MxIconTheme, mx_icon_theme, G_TYPE_OBJECT)
 
@@ -257,6 +258,80 @@ mx_icon_theme_hash (GThemedIcon *icon)
   return g_str_hash (names[0]);
 }
 
+static gboolean
+mx_icon_theme_find_name_cb (gpointer key,
+                            gpointer value,
+                            gpointer user_data)
+{
+  gchar *name1 = (gchar *)value;
+  gchar *name2 = (gchar *)user_data;
+  return g_str_equal (name1, name2);
+}
+
+static void
+mx_icon_theme_load_fallbacks (MxIconTheme *theme,
+                              GKeyFile    *theme_file,
+                              gboolean     root)
+{
+  MxIconThemePrivate *priv = theme->priv;
+  gchar *fallbacks = g_key_file_get_string (theme_file,
+                                            "Icon Theme",
+                                            "Inherits",
+                                            NULL);
+
+  /* If this isn't the root theme, add it to the list of fallbacks */
+  if (!root)
+    priv->theme_fallbacks = g_list_append (priv->theme_fallbacks,
+                                           theme_file);
+
+  /* Check the list of fallbacks in this theme and add any that we haven't
+   * already.
+   */
+  if (fallbacks)
+    {
+      gint i = 0;
+      gint fallbacks_len = strlen (fallbacks);
+
+      g_strdelimit (fallbacks, ",", '\0');
+
+      while (i < fallbacks_len)
+        {
+          gchar *fallback = fallbacks + i;
+          i += strlen (fallback) + 1;
+
+          /* Skip hicolor, we keep it loaded all the time */
+          if (g_str_equal (fallback, "hicolor"))
+            continue;
+
+          /* Skip if we've already loaded this theme */
+          if (g_hash_table_find (priv->theme_path_hash,
+                                 mx_icon_theme_find_name_cb,
+                                 fallback))
+            continue;
+
+          /* Load this theme and store itself and its fallbacks in the
+           * list of fallbacks.
+           */
+          theme_file = mx_icon_theme_load_theme (theme, fallback);
+          if (theme_file)
+            mx_icon_theme_load_fallbacks (theme, theme_file, FALSE);
+        }
+      g_free (fallbacks);
+    }
+}
+
+static void
+mx_icon_theme_changed_cb (MxSettings  *settings,
+                          GParamSpec  *pspec,
+                          MxIconTheme *self)
+{
+  gchar *theme;
+
+  g_object_get (settings, "icon-theme", &theme, NULL);
+  mx_icon_theme_set_theme_name (self, theme);
+  g_free (theme);
+}
+
 static void
 mx_icon_theme_init (MxIconTheme *self)
 {
@@ -306,10 +381,16 @@ mx_icon_theme_init (MxIconTheme *self)
     g_warning ("Error loading fallback icon theme");
 
   theme = g_getenv ("MX_ICON_THEME");
-  if (!theme)
-    theme = "moblin";
+  if (theme)
+    mx_icon_theme_set_theme_name (self, theme);
+  else
+    {
+      MxSettings *settings = mx_settings_get_default ();
+      g_signal_connect (settings, "notify::icon-theme",
+                        G_CALLBACK (mx_icon_theme_changed_cb), self);
+      mx_icon_theme_changed_cb (settings, NULL, self);
+    }
 
-  mx_icon_theme_set_theme_name (self, theme);
 }
 
 /**
@@ -375,7 +456,6 @@ void
 mx_icon_theme_set_theme_name (MxIconTheme *theme,
                               const gchar *theme_name)
 {
-  gchar *fallbacks;
   MxIconThemePrivate *priv;
 
   g_return_if_fail (MX_IS_ICON_THEME (theme));
@@ -419,32 +499,7 @@ mx_icon_theme_set_theme_name (MxIconTheme *theme,
     }
 
   /* Load fallbacks */
-  fallbacks = g_key_file_get_string (priv->theme_file,
-                                     "Icon Theme",
-                                     "Inherits",
-                                     NULL);
-  if (fallbacks)
-    {
-      gint i = 0;
-      gint fallbacks_len = strlen (fallbacks);
-      while (i < fallbacks_len)
-        {
-          GKeyFile *theme_file;
-
-          gchar *fallback = fallbacks + i;
-          i += strlen (fallback) + 1;
-
-          /* Skip hicolor, we keep it loaded all the time */
-          if (g_str_equal (fallback, "hicolor"))
-            continue;
-
-          theme_file = mx_icon_theme_load_theme (theme, fallback);
-          if (theme_file)
-            priv->theme_fallbacks = g_list_append (priv->theme_fallbacks,
-                                                   theme_file);
-        }
-      g_free (fallbacks);
-    }
+  mx_icon_theme_load_fallbacks (theme, priv->theme_file, TRUE);
 
   g_object_notify (G_OBJECT (theme), "theme-name");
 }
@@ -489,7 +544,8 @@ static GList *
 mx_icon_theme_theme_load_icon (MxIconTheme *self,
                                GKeyFile    *theme_file,
                                const gchar *icon,
-                               GIcon       *store_icon)
+                               GIcon       *store_icon,
+                               gboolean     store_fail)
 {
   gchar *dirs;
   const gchar *theme;
@@ -653,7 +709,7 @@ mx_icon_theme_theme_load_icon (MxIconTheme *self,
       g_free (dirs);
     }
 
-  if (data)
+  if (data || store_fail)
     {
       data = g_list_reverse (data);
       if (!store_icon)
@@ -678,7 +734,7 @@ mx_icon_theme_load_icon (MxIconTheme *theme,
   /* Try the set theme */
   if (priv->theme_file &&
       (data = mx_icon_theme_theme_load_icon (theme, priv->theme_file,
-                                             icon_name, store_icon)))
+                                             icon_name, store_icon, FALSE)))
     return data;
 
   /* Try the inherited themes */
@@ -686,14 +742,14 @@ mx_icon_theme_load_icon (MxIconTheme *theme,
     {
       GKeyFile *theme_file = f->data;
       if ((data = mx_icon_theme_theme_load_icon (theme, theme_file, icon_name,
-                                                 store_icon)))
+                                                 store_icon, FALSE)))
         return data;
     }
 
   /* Try the hicolor theme */
   if (priv->hicolor_file &&
       (data = mx_icon_theme_theme_load_icon (theme, priv->hicolor_file,
-                                             icon_name, store_icon)))
+                                             icon_name, store_icon, TRUE)))
     return data;
 
   return NULL;
@@ -740,20 +796,25 @@ mx_icon_theme_get_icons (MxIconTheme *theme,
     {
       /* See if we've loaded this before */
       GIcon *single_icon = g_themed_icon_new (names[i]);
-      data = g_hash_table_lookup (priv->icon_hash, single_icon);
+      gboolean success = g_hash_table_lookup_extended (priv->icon_hash,
+                                                       single_icon,
+                                                       NULL,
+                                                       (gpointer *)&data);
+
       g_object_unref (single_icon);
 
       /* Found in cache on first hit, break */
-      if (data && (i == 0))
+      if (success && (i == 0))
         break;
 
       /* Found in cache after searching the disk, store again as a new icon */
-      if (data)
+      if (success)
         {
           /* If we found this as a fallback, store it so we don't look on
            * disk again.
            */
-          data = mx_icon_theme_copy_data_list (data);
+          if (data)
+            data = mx_icon_theme_copy_data_list (data);
           g_hash_table_insert (priv->icon_hash, g_object_ref (icon), data);
 
           break;
@@ -906,6 +967,15 @@ mx_icon_theme_get_search_paths (MxIconTheme *theme)
   return theme->priv->search_paths;
 }
 
+/**
+ * mx_icon_theme_set_search_paths:
+ * @theme: a #MxIconTheme
+ * @paths: (element-type utf8): a list of search paths
+ *
+ * Sets the directories the #MxIconTheme will search in to find icons.
+ * By default, it will look in the default system and local icon
+ * directories.
+ */
 void
 mx_icon_theme_set_search_paths (MxIconTheme *theme,
                                 const GList *paths)
