@@ -55,13 +55,19 @@ G_DEFINE_TYPE_WITH_CODE (MxIcon, mx_icon, MX_TYPE_WIDGET,
 
 struct _MxIconPrivate
 {
+  guint         icon_set         : 1;
+  guint         size_set         : 1;
+  guint         is_content_image : 1;
+
   ClutterActor *icon_texture;
-  gboolean      is_content_image;
 
   gchar        *icon_name;
+  gchar        *icon_suffix;
   gint          icon_size;
 };
 
+static void mx_icon_update (MxIcon *icon);
+
 static void
 mx_stylable_iface_init (MxStylableIface *iface)
 {
@@ -87,6 +93,13 @@ mx_stylable_iface_init (MxStylableIface *iface)
                                    G_PARAM_READWRITE);
       mx_stylable_iface_install_property (iface, MX_TYPE_ICON, pspec);
 
+      pspec = g_param_spec_string ("x-mx-icon-suffix",
+                                   "Icon suffix",
+                                   "Suffix to append to the icon name",
+                                   NULL,
+                                   G_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_ICON, pspec);
+
       pspec = g_param_spec_int ("x-mx-icon-size",
                                 "Icon size",
                                 "Size to use for icon",
@@ -144,9 +157,18 @@ mx_icon_get_property (GObject    *gobject,
     }
 }
 
+static void
+mx_icon_notify_theme_name_cb (MxIconTheme *theme,
+                              GParamSpec  *pspec,
+                              MxIcon      *self)
+{
+  mx_icon_update (self);
+}
+
 static void
 mx_icon_dispose (GObject *gobject)
 {
+  MxIconTheme *theme;
   MxIconPrivate *priv = MX_ICON (gobject)->priv;
 
   if (priv->icon_texture)
@@ -155,9 +177,27 @@ mx_icon_dispose (GObject *gobject)
       priv->icon_texture = NULL;
     }
 
+  if ((theme = mx_icon_theme_get_default ()))
+    {
+      g_signal_handlers_disconnect_by_func (mx_icon_theme_get_default (),
+                                            mx_icon_notify_theme_name_cb,
+                                            gobject);
+    }
+
   G_OBJECT_CLASS (mx_icon_parent_class)->dispose (gobject);
 }
 
+static void
+mx_icon_finalize (GObject *gobject)
+{
+  MxIconPrivate *priv = MX_ICON (gobject)->priv;
+
+  g_free (priv->icon_name);
+  g_free (priv->icon_suffix);
+
+  G_OBJECT_CLASS (mx_icon_parent_class)->finalize (gobject);
+}
+
 static void
 mx_icon_get_preferred_height (ClutterActor *actor,
                               gfloat        for_width,
@@ -312,6 +352,7 @@ mx_icon_class_init (MxIconClass *klass)
   object_class->get_property = mx_icon_get_property;
   object_class->set_property = mx_icon_set_property;
   object_class->dispose = mx_icon_dispose;
+  object_class->finalize = mx_icon_finalize;
 
   actor_class->get_preferred_height = mx_icon_get_preferred_height;
   actor_class->get_preferred_width = mx_icon_get_preferred_width;
@@ -339,7 +380,12 @@ mx_icon_update (MxIcon *icon)
 {
   MxIconPrivate *priv = icon->priv;
 
-  priv->is_content_image = FALSE;
+  if (priv->is_content_image)
+    {
+      priv->is_content_image = FALSE;
+      g_signal_connect (mx_icon_theme_get_default (), "notify::theme-name",
+                        G_CALLBACK (mx_icon_notify_theme_name_cb), icon);
+    }
 
   /* Get rid of the old one */
   if (priv->icon_texture)
@@ -351,13 +397,23 @@ mx_icon_update (MxIcon *icon)
   /* Try to lookup the new one */
   if (priv->icon_name)
     {
+      gchar *icon_name;
       MxIconTheme *theme = mx_icon_theme_get_default ();
+
+      icon_name = g_strconcat (priv->icon_name, priv->icon_suffix, NULL);
       priv->icon_texture = (ClutterActor *)
-        mx_icon_theme_lookup_texture (theme, priv->icon_name, priv->icon_size);
+        mx_icon_theme_lookup_texture (theme, icon_name, priv->icon_size);
+      g_free (icon_name);
+
+      /* If the icon is missing, use the image-missing icon */
+      if (!priv->icon_texture)
+        priv->icon_texture = (ClutterActor *)
+          mx_icon_theme_lookup_texture (theme,
+                                        "image-missing",
+                                        priv->icon_size);
+
       if (priv->icon_texture)
         clutter_actor_set_parent (priv->icon_texture, CLUTTER_ACTOR (icon));
-      else
-        g_warning ("Failed to lookup icon '%s'", priv->icon_name);
     }
 
   clutter_actor_queue_relayout (CLUTTER_ACTOR (icon));
@@ -372,12 +428,14 @@ mx_icon_style_changed_cb (MxWidget *widget)
   MxBorderImage *content_image = NULL;
   gboolean changed = FALSE;
   gchar *icon_name = NULL;
+  gchar *icon_suffix = NULL;
   gint icon_size = -1;
 
   mx_stylable_get (MX_STYLABLE (widget),
                    "x-mx-content-image", &content_image,
                    "x-mx-icon-name", &icon_name,
                    "x-mx-icon-size", &icon_size,
+                   "x-mx-icon-suffix", &icon_suffix,
                    NULL);
 
   /* Content-image overrides drawing of the icon, so
@@ -388,12 +446,19 @@ mx_icon_style_changed_cb (MxWidget *widget)
       GError *error = NULL;
 
       priv->is_content_image = TRUE;
+      g_signal_handlers_disconnect_by_func (mx_icon_theme_get_default (),
+                                            mx_icon_notify_theme_name_cb,
+                                            self);
 
       if (priv->icon_texture)
-        clutter_actor_destroy (priv->icon_texture);
+        {
+          clutter_actor_destroy (priv->icon_texture);
+          priv->icon_texture = NULL;
+        }
 
-      priv->icon_texture = clutter_texture_new_from_file (content_image->uri,
-                                                          &error);
+      if (content_image->uri)
+        priv->icon_texture = clutter_texture_new_from_file (content_image->uri,
+                                                            &error);
       if (priv->icon_texture)
         clutter_actor_set_parent (priv->icon_texture, CLUTTER_ACTOR (widget));
 
@@ -409,8 +474,8 @@ mx_icon_style_changed_cb (MxWidget *widget)
       return;
     }
 
-  if (icon_name && (!priv->icon_name ||
-                    !g_str_equal (icon_name, priv->icon_name)))
+  if (icon_name && !priv->icon_set &&
+      (!priv->icon_name || !g_str_equal (icon_name, priv->icon_name)))
     {
       g_free (priv->icon_name);
       priv->icon_name = g_strdup (icon_name);
@@ -419,7 +484,7 @@ mx_icon_style_changed_cb (MxWidget *widget)
       g_object_notify (G_OBJECT (self), "icon-name");
     }
 
-  if ((icon_size > 0) && (priv->icon_size != icon_size))
+  if ((icon_size > 0) && !priv->size_set && (priv->icon_size != icon_size))
     {
       priv->icon_size = icon_size;
       changed = TRUE;
@@ -427,6 +492,18 @@ mx_icon_style_changed_cb (MxWidget *widget)
       g_object_notify (G_OBJECT (self), "icon-size");
     }
 
+  if ((icon_suffix != priv->icon_suffix) &&
+      (!icon_suffix || !priv->icon_suffix ||
+       !g_str_equal (icon_suffix, priv->icon_suffix)))
+    {
+      g_free (priv->icon_suffix);
+      priv->icon_suffix = icon_suffix;
+
+      changed = TRUE;
+    }
+  else
+    g_free (icon_suffix);
+
   if (changed)
     mx_icon_update (self);
 }
@@ -443,6 +520,10 @@ mx_icon_init (MxIcon *self)
 
   /* make sure we are not reactive */
   clutter_actor_set_reactive (CLUTTER_ACTOR (self), FALSE);
+
+  /* Reload the icon when the theme changes */
+  g_signal_connect (mx_icon_theme_get_default (), "notify::theme-name",
+                    G_CALLBACK (mx_icon_notify_theme_name_cb), self);
 }
 
 /**
@@ -477,10 +558,22 @@ mx_icon_set_icon_name (MxIcon      *icon,
 
   priv = icon->priv;
 
+  /* Unset the icon name if necessary */
+  if (!icon_name)
+    {
+      if (priv->icon_set)
+        {
+          priv->icon_set = FALSE;
+          mx_stylable_style_changed (MX_STYLABLE (icon), MX_STYLE_CHANGED_NONE);
+        }
+
+      return;
+    }
+
+  priv->icon_set = TRUE;
+
   /* Check if there's no change */
-  if ((!priv->icon_name && !icon_name) ||
-      (priv->icon_name && icon_name &&
-       g_str_equal (priv->icon_name, icon_name)))
+  if (priv->icon_name && g_str_equal (priv->icon_name, icon_name))
     return;
 
   g_free (priv->icon_name);
@@ -498,6 +591,7 @@ mx_icon_get_icon_size (MxIcon *icon)
 
   return icon->priv->icon_size;
 }
+
 void
 mx_icon_set_icon_size (MxIcon *icon,
                        gint    size)
@@ -505,13 +599,26 @@ mx_icon_set_icon_size (MxIcon *icon,
   MxIconPrivate *priv;
 
   g_return_if_fail (MX_IS_ICON (icon));
-  g_return_if_fail (size > 0);
 
   priv = icon->priv;
-  if (priv->icon_size != size)
+
+  if (size < 0)
+    {
+      if (priv->size_set)
+        {
+          priv->size_set = FALSE;
+          mx_stylable_style_changed (MX_STYLABLE (icon), MX_STYLE_CHANGED_NONE);
+        }
+
+      return;
+    }
+  else if (priv->icon_size != size)
     {
       priv->icon_size = size;
       mx_icon_update (icon);
+
       g_object_notify (G_OBJECT (icon), "icon-size");
     }
+
+  priv->size_set = TRUE;
 }
