@@ -258,6 +258,25 @@ mx_scroll_bar_unmap (ClutterActor *actor)
     clutter_actor_unmap (priv->handle);
 }
 
+static void
+mx_scroll_bar_apply_style (MxWidget *widget,
+                           MxStyle  *style)
+{
+  MxScrollBarPrivate *priv = MX_SCROLL_BAR (widget)->priv;
+
+  if (priv->bw_stepper != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->bw_stepper), style);
+
+  if (priv->fw_stepper != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->fw_stepper), style);
+
+  if (priv->trough != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->trough), style);
+
+  if (priv->handle != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->handle), style);
+}
+
 static void
 mx_scroll_bar_get_preferred_width (ClutterActor *actor,
                                    gfloat        for_height,
@@ -427,7 +446,7 @@ mx_scroll_bar_allocate (ClutterActor          *actor,
 
   if (priv->adjustment)
     {
-      gfloat handle_size, position, avail_size;
+      gfloat handle_size, position, avail_size, handle_pos;
       gdouble value, lower, upper, page_size, increment;
       ClutterActorBox handle_box = { 0, };
       guint min_size, max_size;
@@ -466,10 +485,13 @@ mx_scroll_bar_allocate (ClutterActor          *actor,
           handle_size = CLAMP (handle_size, min_size, max_size);
 
           handle_box.x1 = x;
-          handle_box.y1 = bw_box.y2 + position * (avail_size - handle_size);
+          handle_pos = bw_box.y2 + position * (avail_size - handle_size);
+          handle_box.y1 = CLAMP (handle_pos,
+                                 bw_box.y2, fw_box.y1 - min_size);
 
           handle_box.x2 = handle_box.x1 + width;
-          handle_box.y2 = handle_box.y1 + handle_size;
+          handle_box.y2 = CLAMP (handle_pos + handle_size,
+                                 bw_box.y2 + min_size, fw_box.y1);
         }
       else
         {
@@ -477,10 +499,13 @@ mx_scroll_bar_allocate (ClutterActor          *actor,
           handle_size = increment * avail_size;
           handle_size = CLAMP (handle_size, min_size, max_size);
 
-          handle_box.x1 = bw_box.x2 + position * (avail_size - handle_size);
+          handle_pos = bw_box.x2 + position * (avail_size - handle_size);
+          handle_box.x1 = CLAMP (handle_pos,
+                                 bw_box.x2, fw_box.x1 - min_size);
           handle_box.y1 = y;
 
-          handle_box.x2 = handle_box.x1 + handle_size;
+          handle_box.x2 = CLAMP (handle_pos + handle_size,
+                                 bw_box.x2 + min_size, fw_box.x1);
           handle_box.y2 = handle_box.y1 + height;
         }
 
@@ -594,6 +619,7 @@ mx_scroll_bar_class_init (MxScrollBarClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  MxWidgetClass *widget_class = MX_WIDGET_CLASS (klass);
   GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (MxScrollBarPrivate));
@@ -612,6 +638,8 @@ mx_scroll_bar_class_init (MxScrollBarClass *klass)
   actor_class->map            = mx_scroll_bar_map;
   actor_class->unmap          = mx_scroll_bar_unmap;
 
+  widget_class->apply_style = mx_scroll_bar_apply_style;
+
   g_object_class_install_property
                  (object_class,
                  PROP_ADJUSTMENT,
@@ -1170,7 +1198,7 @@ mx_scroll_bar_set_adjustment (MxScrollBar  *bar,
 
 /**
  * mx_scroll_bar_get_adjustment:
- * @bar: a #MxScrollbar
+ * @bar: a #MxScrollBar
  *
  * Gets the adjustment object that stores the current position
  * of the scrollbar.
