@@ -33,12 +33,16 @@
 #include "mx-progress-bar-fill.h"
 #include "mx-button.h"
 #include "mx-frame.h"
+#include "mx-focusable.h"
 
 static void mx_stylable_iface_init (MxStylableIface *iface);
+static void mx_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxSlider, mx_slider, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
-                                                mx_stylable_iface_init))
+                                                mx_stylable_iface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                mx_focusable_iface_init))
 
 
 #define SLIDER_PRIVATE(o)                         \
@@ -80,6 +84,45 @@ enum
   PROP_VALUE
 };
 
+/* MxFocusable interface */
+
+MxFocusable*
+mx_slider_move_focus (MxFocusable      *focusable,
+                      MxFocusDirection  direction,
+                      MxFocusable      *old_focus)
+{
+  MxSliderPrivate *priv = MX_SLIDER (focusable)->priv;
+
+  mx_stylable_set_style_pseudo_class (MX_STYLABLE (priv->handle), "");
+  mx_stylable_set_style_pseudo_class (MX_STYLABLE (priv->trough), "");
+
+  return NULL;
+}
+
+MxFocusable*
+mx_slider_accept_focus (MxFocusable *focusable,
+                        MxFocusHint  hint)
+{
+  MxSliderPrivate *priv = MX_SLIDER (focusable)->priv;
+
+  clutter_actor_grab_key_focus (CLUTTER_ACTOR (focusable));
+
+  mx_stylable_set_style_pseudo_class (MX_STYLABLE (priv->handle), "focus");
+  mx_stylable_set_style_pseudo_class (MX_STYLABLE (priv->trough), "focus");
+
+  return focusable;
+}
+
+
+static void
+mx_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->move_focus = mx_slider_move_focus;
+  iface->accept_focus = mx_slider_accept_focus;
+}
+
+
+
 static void
 drag_handle (MxSlider *bar,
              gfloat    x,
@@ -322,11 +365,11 @@ mx_slider_paint (ClutterActor *actor)
 
   clutter_actor_paint (priv->trough_bg);
 
+  clutter_actor_paint (priv->trough);
+
   if (priv->value)
     clutter_actor_paint (priv->fill);
 
-  clutter_actor_paint (priv->trough);
-
   clutter_actor_paint (priv->handle);
 }
 
@@ -426,11 +469,10 @@ mx_slider_allocate_fill_handle (MxSlider               *self,
   handle_box.x1 = fill_box.x2 - handle_width_2;
   handle_box.x2 = handle_box.x1 + priv->handle_width;
 
-  /* If the handle height is unset, or the handle height is greater than the
-   * trough height, we want the handle to occupy all available space.
+  /* If the handle height is unset, occupy all available space.
    * Otherwise we want it to be centred in the trough.
    */
-  if ((priv->handle_height == 0) || (priv->handle_height > priv->trough_height))
+  if (priv->handle_height == 0)
     {
       handle_box.y1 = padding.top;
       handle_box.y2 = (box->y2 - box->y1) - padding.bottom;
@@ -492,8 +534,8 @@ mx_slider_allocate (ClutterActor           *actor,
   else
     {
       trough_box.x1 = padding.left;
-      trough_box.y1 = (box->y2 - box->y1 - padding.bottom - padding.top -
-                       priv->trough_height) / 2;
+      trough_box.y1 = (int) ((box->y2 - box->y1 - padding.bottom - padding.top -
+                       priv->trough_height) / 2);
       trough_box.x2 = (box->x2 - box->x1) - padding.right;
       trough_box.y2 = trough_box.y1 + priv->trough_height;
     }
@@ -536,6 +578,25 @@ mx_slider_unmap (ClutterActor *actor)
   clutter_actor_unmap (priv->handle);
 }
 
+static void
+mx_slider_apply_style (MxWidget *widget,
+                       MxStyle  *style)
+{
+  MxSliderPrivate *priv = MX_SLIDER (widget)->priv;
+
+  if (priv->trough_bg != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->trough_bg), style);
+
+  if (priv->fill != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->fill), style);
+
+  if (priv->trough != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->trough), style);
+
+  if (priv->handle != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->handle), style);
+}
+
 /*
  * GObject overloading
  */
@@ -622,11 +683,37 @@ mx_slider_dispose (GObject *object)
   G_OBJECT_CLASS (mx_slider_parent_class)->dispose (object);
 }
 
+static gboolean
+mx_slider_key_press_event (ClutterActor    *actor,
+                           ClutterKeyEvent *event)
+{
+  gdouble value;
+
+  value = mx_slider_get_value (MX_SLIDER (actor));
+
+  switch (event->keyval)
+    {
+    case CLUTTER_Left :
+      mx_slider_set_value (MX_SLIDER (actor), MAX (value - 0.1, 0));
+      return TRUE;
+
+    case CLUTTER_Right :
+      mx_slider_set_value (MX_SLIDER (actor), MIN (value + 0.1, 1));
+      return TRUE;
+
+    default:
+      break;
+    }
+
+  return FALSE;
+}
+
 static void
 mx_slider_class_init (MxSliderClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  MxWidgetClass *widget_class = MX_WIDGET_CLASS (klass);
   GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (MxSliderPrivate));
@@ -642,6 +729,9 @@ mx_slider_class_init (MxSliderClass *klass)
   actor_class->allocate = mx_slider_allocate;
   actor_class->map = mx_slider_map;
   actor_class->unmap = mx_slider_unmap;
+  actor_class->key_press_event = mx_slider_key_press_event;
+
+  widget_class->apply_style = mx_slider_apply_style;
 
   pspec = g_param_spec_double ("value",
                                "Value",
