@@ -45,6 +45,7 @@
 #include "mx-marshal.h"
 #include "mx-private.h"
 #include "mx-stylable.h"
+#include "mx-settings.h"
 
 enum
 {
@@ -66,6 +67,8 @@ static GQuark quark_style              = 0;
 
 static guint stylable_signals[LAST_SIGNAL] = { 0, };
 
+static void mx_stylable_property_changed_notify (MxStylable *stylable);
+
 static void
 mx_stylable_notify_dispatcher (GObject     *gobject,
                                guint        n_pspecs,
@@ -147,13 +150,13 @@ mx_stylable_base_init (gpointer g_iface)
   /**
    * MxStylable::style-changed:
    * @stylable: the #MxStylable that received the signal
-   * @old_style: the previously set #MxStyle for @stylable
+   * @flags: the #MxStyleChangedFlags associated with the signal
    *
    * The ::style-changed signal is emitted each time one of the style
    * properties have changed.
    */
   stylable_signals[STYLE_CHANGED] =
-    g_signal_new (I_("style-changed"),
+    g_signal_new (g_intern_static_string ("style-changed"),
                   iface_type,
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (MxStylableIface, style_changed),
@@ -163,7 +166,7 @@ mx_stylable_base_init (gpointer g_iface)
 
 #if 0
   stylable_signals[STYLE_NOTIFY] =
-    g_signal_new (I_("style-notify"),
+    g_signal_new (g_intern_static_string ("style-notify"),
                   iface_type,
                   G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED
                   | G_SIGNAL_NO_HOOKS | G_SIGNAL_ACTION,
@@ -189,13 +192,38 @@ mx_stylable_get_type (void)
       };
 
       our_type = g_type_register_static (G_TYPE_INTERFACE,
-                                         I_("MxStylable"),
+                                         g_intern_static_string ("MxStylable"),
                                          &stylable_info, 0);
     }
 
   return our_type;
 }
 
+gchar *
+_mx_stylable_get_style_string (MxStylable *stylable)
+{
+  GType type_id;
+  const gchar *type, *id, *class, *pseudo_class;
+
+  /* Create a string that contains all the properties of a
+   * Stylable that can be matched against in the CSS.
+   */
+  type_id = G_OBJECT_CLASS_TYPE (G_OBJECT_GET_CLASS (stylable));
+  type = g_type_name (type_id);
+
+  id = clutter_actor_get_name (CLUTTER_ACTOR (stylable));
+
+  class = mx_stylable_get_style_class (stylable);
+
+  pseudo_class = mx_stylable_get_style_pseudo_class (stylable);
+
+  return g_strdup_printf ("%s#%s.%s:%s",
+                          type,
+                          id ? id : "",
+                          class ? class : "",
+                          pseudo_class ? pseudo_class : "");
+}
+
 #if 0
 void
 mx_stylable_freeze_notify (MxStylable *stylable)
@@ -510,6 +538,63 @@ mx_stylable_get (MxStylable  *stylable,
   va_end (args);
 }
 
+static gboolean
+_set_from_xsettings (GParamSpec *pspec,
+                     GValue     *value)
+{
+  gchar *font_string;
+  gint font_size;
+  PangoFontDescription *descr;
+  MxSettings *settings = mx_settings_get_default ();
+  gboolean result = FALSE;
+
+  if (!strcmp (pspec->name, "font-family")
+      || !strcmp (pspec->name, "font-size"))
+    {
+      const gchar *fn;
+      PangoFontMask set_fields;
+
+      g_object_get (settings, "font-name", &font_string, NULL);
+
+      if (!font_string)
+        return FALSE;
+
+      descr = pango_font_description_from_string (font_string);
+
+      set_fields = pango_font_description_get_set_fields (descr);
+
+      /* font name */
+      if ((set_fields & PANGO_FONT_MASK_FAMILY)
+          && !strcmp (pspec->name, "font-family")
+          && G_VALUE_HOLDS_STRING (value))
+        {
+          fn = pango_font_description_get_family (descr);
+          g_value_set_string (value, fn);
+          result = TRUE;
+        }
+      /* font size */
+      else if ((set_fields & PANGO_FONT_MASK_SIZE)
+               && !strcmp (pspec->name, "font-size")
+               && G_VALUE_HOLDS_INT (value))
+        {
+          font_size = pango_font_description_get_size (descr) / PANGO_SCALE;
+          if (!pango_font_description_get_size_is_absolute (descr))
+            {
+              ClutterBackend *backend = clutter_get_default_backend ();
+              gdouble res = clutter_backend_get_resolution (backend);
+              font_size = font_size * res / 72.0;
+            }
+          g_value_set_int (value, font_size);
+          result = TRUE;
+        }
+
+      pango_font_description_free (descr);
+      descr = NULL;
+    }
+
+  return result;
+}
+
 /**
  * mx_stylable_get_default_value:
  * @stylable: a #MxStylable
@@ -548,7 +633,11 @@ mx_stylable_get_default_value (MxStylable  *stylable,
     }
 
   g_value_init (value_out, G_PARAM_SPEC_VALUE_TYPE (pspec));
-  g_param_value_set_default (pspec, value_out);
+
+  /* default font values come from xsettings if possible */
+  if (!_set_from_xsettings (pspec, value_out))
+    g_param_value_set_default (pspec, value_out);
+
   return TRUE;
 }
 
@@ -575,6 +664,20 @@ mx_stylable_get_style (MxStylable *stylable)
   return g_object_get_data (G_OBJECT (stylable), "mx-stylable-style");
 }
 
+typedef struct
+{
+  GObject *instance;
+  gulong   handler_id;
+} SignalData;
+
+static void
+disconnect_style_changed_signal (SignalData *data)
+{
+  g_signal_handler_disconnect (data->instance, data->handler_id);
+  g_object_unref (data->instance);
+  g_slice_free (SignalData, data);
+}
+
 /**
  * mx_stylable_set_style:
  * @stylable: a #MxStylable
@@ -584,7 +687,7 @@ mx_stylable_get_style (MxStylable *stylable)
  *
  * The #MxStylable will take ownership of the passed #MxStyle.
  *
- * After the #MxStle has been set, the MxStylable::style-set signal
+ * After the #MxStyle has been set, the MxStylable::style-set signal
  * will be emitted.
  */
 void
@@ -592,28 +695,31 @@ mx_stylable_set_style (MxStylable *stylable,
                        MxStyle    *style)
 {
   MxStylableIface *iface;
-  MxStyle *old_style;
+  SignalData *data;
 
   g_return_if_fail (MX_IS_STYLABLE (stylable));
   g_return_if_fail (MX_IS_STYLE (style));
 
   iface = MX_STYLABLE_GET_IFACE (stylable);
 
-  old_style = mx_stylable_get_style (stylable);
-  g_object_ref (old_style);
-
   if (iface->set_style)
     iface->set_style (stylable, style);
-  else
-    {
-      g_object_set_qdata_full (G_OBJECT (stylable),
-                               quark_style,
-                               g_object_ref_sink (style),
-                               g_object_unref);
-    }
 
-  g_signal_emit (stylable, stylable_signals[STYLE_CHANGED], 0, old_style);
-  g_object_unref (old_style);
+  /* connect to the "changed" signal of the MxStyle and take a reference to
+   * the style */
+  data = g_slice_new (SignalData);
+  data->instance = g_object_ref_sink (style);
+  data->handler_id =
+    g_signal_connect_swapped (style, "changed",
+                              G_CALLBACK (mx_stylable_property_changed_notify),
+                              stylable);
+
+  g_object_set_qdata_full (G_OBJECT (stylable),
+                           quark_style,
+                           data,
+                           (GDestroyNotify) disconnect_style_changed_signal);
+
+  mx_stylable_style_changed (stylable, MX_STYLE_CHANGED_INVALIDATE_CACHE);
 
   g_object_notify (G_OBJECT (stylable), "style");
 }
@@ -724,7 +830,7 @@ mx_stylable_set_style_class (MxStylable  *stylable,
 static void
 mx_stylable_property_changed_notify (MxStylable *stylable)
 {
-  mx_stylable_style_changed (stylable, MX_STYLE_CHANGED_NONE);
+  mx_stylable_style_changed (stylable, MX_STYLE_CHANGED_INVALIDATE_CACHE);
 }
 
 static void
@@ -736,7 +842,8 @@ mx_stylable_parent_set_notify (ClutterActor *actor,
   /* check the actor has a new parent */
   if (new_parent)
     {
-      mx_stylable_style_changed (MX_STYLABLE (actor), MX_STYLE_CHANGED_NONE);
+      mx_stylable_style_changed (MX_STYLABLE (actor),
+                                 MX_STYLE_CHANGED_INVALIDATE_CACHE);
     }
 }
 
@@ -758,10 +865,14 @@ mx_stylable_style_changed_internal (MxStylable          *stylable,
 {
 
   /* don't update stylables until they are mapped (unless ensure is set) */
-  if (!CLUTTER_ACTOR_IS_MAPPED (CLUTTER_ACTOR (stylable)) &&
+  if (G_LIKELY (CLUTTER_IS_ACTOR (stylable)) &&
+      !CLUTTER_ACTOR_IS_MAPPED (CLUTTER_ACTOR (stylable)) &&
       !(flags & MX_STYLE_CHANGED_FORCE))
     return;
 
+  if (flags & MX_STYLE_CHANGED_INVALIDATE_CACHE)
+    _mx_style_invalidate_cache (stylable);
+
   g_signal_emit (stylable, stylable_signals[STYLE_CHANGED], 0, flags);
 
   /* propagate the style-changed signal to children, since their style may
@@ -818,6 +929,7 @@ mx_stylable_connect_change_notifiers (MxStylable *stylable)
                     G_CALLBACK (mx_stylable_property_changed_notify), NULL);
   g_signal_connect (stylable, "notify::style-pseudo-class",
                     G_CALLBACK (mx_stylable_property_changed_notify), NULL);
+
 }
 
 void
