@@ -42,6 +42,7 @@
 #include "mx-style.h"
 #include "mx-enum-types.h"
 #include "mx-types.h"
+#include "mx-private.h"
 
 enum
 {
@@ -55,6 +56,13 @@ enum
 
 #define MX_STYLE_ERROR g_style_error_quark ()
 
+#define MX_STYLE_CACHE g_style_cache_quark ()
+
+/* Default cache size - 6 should probably be enough,
+ * e.g. normal, hover, active, checked, focus + 1 extra
+ */
+#define MX_STYLE_CACHE_SIZE 6
+
 typedef struct {
   GType value_type;
   gchar *value_name;
@@ -82,6 +90,12 @@ g_style_error_quark (void)
   return g_quark_from_static_string ("mx-style-error-quark");
 }
 
+static GQuark
+g_style_cache_quark (void)
+{
+  return g_quark_from_static_string ("mx-style-cache-quark");
+}
+
 static gboolean
 mx_style_real_load_from_file (MxStyle    *style,
                                 const gchar  *filename,
@@ -266,7 +280,19 @@ mx_style_transform_css_value (MxStyleSheetValue *css_value,
       g_value_init (value, pspec->value_type);
 
       if (css_value->string)
-        g_value_set_int (value, atoi (css_value->string));
+        {
+          gint number = atoi (css_value->string);
+
+          if (g_str_equal (g_param_spec_get_name (pspec), "font-size") &&
+              g_str_has_suffix (css_value->string, "pt"))
+            {
+              ClutterBackend *backend = clutter_get_default_backend ();
+              gdouble res = clutter_backend_get_resolution (backend);
+              number = number * res / 72.0;
+            }
+
+          g_value_set_int (value, number);
+        }
       else
         g_value_set_int (value, 0);
     }
@@ -337,6 +363,7 @@ mx_style_transform_css_value (MxStyleSheetValue *css_value,
     }
 }
 
+
 static const gchar*
 mx_style_normalize_property_name (const gchar *name)
 {
@@ -354,6 +381,129 @@ mx_style_normalize_property_name (const gchar *name)
     return name;
 }
 
+/* A style cache entry is the unique string representing all the properties
+ * that can be matched against in CSS, and the matched properties themselves.
+ */
+typedef struct
+{
+  gchar      *style_string;
+  GHashTable *properties;
+} MxStyleCacheEntry;
+
+static void
+mx_style_object_died_cb (GArray  *cache,
+                         GObject *old_object)
+{
+  gint i;
+
+  /* The object we've added a cache to has died, so free
+   * the cached data.
+   */
+  for (i = 0; i < cache->len; i++)
+    {
+      MxStyleCacheEntry *entry =
+        &g_array_index (cache, MxStyleCacheEntry, i);
+
+      g_hash_table_unref (entry->properties);
+      g_free (entry->style_string);
+    }
+
+  /* We use this function to either destroy the cache, or
+   * empty it, depending on if old_object is set.
+   */
+  if (old_object)
+    g_array_free (cache, TRUE);
+  else
+    g_array_set_size (cache, 0);
+}
+
+void
+_mx_style_invalidate_cache (MxStylable *stylable)
+{
+  GObject *object = G_OBJECT (stylable);
+  GArray *cache = g_object_get_qdata (object, MX_STYLE_CACHE);
+
+  /* Empty the cache */
+  if (cache)
+    mx_style_object_died_cb (cache, NULL);
+}
+
+static GHashTable *
+mx_style_get_style_sheet_properties (MxStyle    *style,
+                                     MxStylable *stylable)
+{
+  GArray *cache;
+  gchar *style_string;
+  GHashTable *properties = NULL;
+  MxStylePrivate *priv = style->priv;
+
+  /* see if we have a cached style and return that if possible */
+  cache = g_object_get_qdata (G_OBJECT (stylable), MX_STYLE_CACHE);
+  style_string = _mx_stylable_get_style_string (stylable);
+
+  if (cache)
+    {
+      /* A cache exists, so check if this particular style-state
+       * has been cached already.
+       */
+      gint i;
+      for (i = 0; i < cache->len; i++)
+        {
+          MxStyleCacheEntry *entry =
+            &g_array_index (cache, MxStyleCacheEntry, i);
+
+          if (!g_strcmp0 (style_string, entry->style_string))
+            {
+              g_free (style_string);
+              properties = entry->properties;
+              break;
+            }
+        }
+    }
+  else
+    {
+      /* This is the first time this stylable has tried to get style
+       * properties, initialise a cache.
+       */
+      cache = g_array_new (FALSE, TRUE, sizeof (MxStyleCacheEntry));
+      g_object_set_qdata (G_OBJECT (stylable), MX_STYLE_CACHE, cache);
+      g_object_weak_ref (G_OBJECT (stylable),
+                         (GWeakNotify)mx_style_object_died_cb, cache);
+    }
+
+  /* No cached style properties were found, so look them up from the
+   * style-sheet and add them to the cache.
+   */
+  if (!properties)
+    {
+      MxStyleCacheEntry entry;
+
+      properties = mx_style_sheet_get_properties (priv->stylesheet,
+                                                  stylable);
+
+      /* Append this to the style cache */
+      entry.style_string = style_string;
+      entry.properties = properties;
+      g_array_append_val (cache, entry);
+
+      /* Shrink the cache if its grown too large */
+      if (cache->len > MX_STYLE_CACHE_SIZE)
+        {
+          gint i;
+          for (i = 0; i < cache->len - MX_STYLE_CACHE_SIZE; i++)
+            {
+              MxStyleCacheEntry *entry_p;
+              entry_p = &g_array_index (cache, MxStyleCacheEntry, i);
+              g_free (entry_p->style_string);
+              g_hash_table_unref (entry_p->properties);
+            }
+          g_array_remove_range (cache, 0, i);
+        }
+    }
+
+  return g_hash_table_ref (properties);
+}
+
 /**
  * mx_style_get_property:
  * @style: the style data store object
@@ -385,21 +535,19 @@ mx_style_get_property (MxStyle    *style,
       MxStyleSheetValue *css_value;
       GHashTable *properties;
 
-      properties = mx_style_sheet_get_properties (priv->stylesheet, stylable);
+      properties = mx_style_get_style_sheet_properties (style, stylable);
 
       css_value = g_hash_table_lookup (properties,
                                        mx_style_normalize_property_name (pspec->name));
 
       if (!css_value)
         {
-          /* get the default value */
-          g_value_init (value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-          g_param_value_set_default (pspec, value);
+          mx_stylable_get_default_value (stylable, pspec->name, value);
         }
       else
         mx_style_transform_css_value (css_value, stylable, pspec, value);
 
-      g_hash_table_destroy (properties);
+      g_hash_table_unref (properties);
     }
 }
 
@@ -436,7 +584,7 @@ mx_style_get_valist (MxStyle     *style,
     {
       GHashTable *properties;
 
-      properties = mx_style_sheet_get_properties (priv->stylesheet, stylable);
+      properties = mx_style_get_style_sheet_properties (style, stylable);
 
       while (name)
         {
@@ -459,9 +607,7 @@ mx_style_get_valist (MxStyle     *style,
 
           if (!css_value)
             {
-              /* get the default value */
-              g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-              g_param_value_set_default (pspec, &value);
+              mx_stylable_get_default_value (stylable, pspec->name, &value);
             }
           else
             mx_style_transform_css_value (css_value, stylable, pspec, &value);
@@ -481,6 +627,7 @@ mx_style_get_valist (MxStyle     *style,
           name = va_arg (va_args, gchar*);
         }
       values_set = TRUE;
+
       g_hash_table_unref (properties);
     }
 
