@@ -95,6 +95,8 @@ struct _MxTablePrivate
 {
   GList *children;
 
+  guint   ignore_css_col_spacing : 1;
+  guint   ignore_css_row_spacing : 1;
   gint    col_spacing;
   gint    row_spacing;
 
@@ -115,10 +117,13 @@ struct _MxTablePrivate
 
 static void mx_container_iface_init (ClutterContainerIface *iface);
 static void mx_focusable_iface_init (MxFocusableIface *iface);
+static void mx_stylable_iface_init (MxStylableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxTable, mx_table, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
                                                 mx_container_iface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
+                                                mx_stylable_iface_init)
                          G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
                                                 mx_focusable_iface_init));
 
@@ -225,25 +230,29 @@ mx_table_move_focus (MxFocusable      *focusable,
 
       while (!focused && row >= 0)
         {
-
           found = mx_table_find_actor_at (table, row, column);
 
-          if (MX_IS_FOCUSABLE (found))
+          if (found)
             {
-              focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
-                                                   MX_FOCUS_HINT_FIRST);
-              if (focused)
+              if (MX_IS_FOCUSABLE (found))
                 {
-                  priv->last_focus = (MxFocusable *)found;
-                  break;
+                  focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
+                                                       MX_FOCUS_HINT_FIRST);
+                  if (focused)
+                    {
+                      priv->last_focus = (MxFocusable *)found;
+                      break;
+                    }
                 }
+
+              child_meta = (MxTableChild *) clutter_container_get_child_meta
+                (CLUTTER_CONTAINER (focusable), found);
+
+              /* row might not be the top row if @found is a spanned actor */
+              row = child_meta->row - 1;
             }
-
-          child_meta = (MxTableChild *) clutter_container_get_child_meta
-            (CLUTTER_CONTAINER (focusable), found);
-
-          /* row might not be the top row if @found is a spanned actor */
-          row = child_meta->row - 1;
+          else
+            row --;
         }
 
       return focused;
@@ -261,21 +270,26 @@ mx_table_move_focus (MxFocusable      *focusable,
         {
           found = mx_table_find_actor_at (table, row, column);
 
-          if (MX_IS_FOCUSABLE (found))
+          if (found)
             {
-              focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
-                                                   MX_FOCUS_HINT_FIRST);
-              if (focused)
+              if (MX_IS_FOCUSABLE (found))
                 {
-                  priv->last_focus = (MxFocusable *)found;
-                  break;
+                  focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
+                                                       MX_FOCUS_HINT_FIRST);
+                  if (focused)
+                    {
+                      priv->last_focus = (MxFocusable *)found;
+                      break;
+                    }
                 }
+
+              child_meta = (MxTableChild *) clutter_container_get_child_meta
+                (CLUTTER_CONTAINER (focusable), found);
+
+              row = child_meta->row + child_meta->row_span;
             }
-
-          child_meta = (MxTableChild *) clutter_container_get_child_meta
-            (CLUTTER_CONTAINER (focusable), found);
-
-          row = child_meta->row + child_meta->row_span;
+          else
+            row ++;
         }
 
       return focused;
@@ -293,22 +307,27 @@ mx_table_move_focus (MxFocusable      *focusable,
         {
           found = mx_table_find_actor_at (table, row, column);
 
-          if (MX_IS_FOCUSABLE (found))
+          if (found)
             {
-              focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
-                                                   MX_FOCUS_HINT_FIRST);
-              if (focused)
+              if (MX_IS_FOCUSABLE (found))
                 {
-                  priv->last_focus = (MxFocusable *)found;
-                  break;
+                  focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
+                                                       MX_FOCUS_HINT_FIRST);
+                  if (focused)
+                    {
+                      priv->last_focus = (MxFocusable *)found;
+                      break;
+                    }
                 }
+
+              child_meta = (MxTableChild *) clutter_container_get_child_meta
+                (CLUTTER_CONTAINER (focusable), found);
+
+              /* col might not be the first column if @found is a spanned actor */
+              column = child_meta->col - 1;
             }
-
-          child_meta = (MxTableChild *) clutter_container_get_child_meta
-            (CLUTTER_CONTAINER (focusable), found);
-
-          /* col might not be the first column if @found is a spanned actor */
-          column = child_meta->col - 1;
+          else
+            column --;
         }
 
       return focused;
@@ -326,21 +345,26 @@ mx_table_move_focus (MxFocusable      *focusable,
         {
           found = mx_table_find_actor_at (table, row, column);
 
-          if (MX_IS_FOCUSABLE (found))
+          if (found)
             {
-              focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
-                                                   MX_FOCUS_HINT_FIRST);
-              if (focused)
+              if (MX_IS_FOCUSABLE (found))
                 {
-                  priv->last_focus = (MxFocusable *)found;
-                  break;
+                  focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
+                                                       MX_FOCUS_HINT_FIRST);
+                  if (focused)
+                    {
+                      priv->last_focus = (MxFocusable *)found;
+                      break;
+                    }
                 }
+
+              child_meta = (MxTableChild *) clutter_container_get_child_meta
+                (CLUTTER_CONTAINER (focusable), found);
+
+              column = child_meta->col + child_meta->col_span;
             }
-
-          child_meta = (MxTableChild *) clutter_container_get_child_meta
-            (CLUTTER_CONTAINER (focusable), found);
-
-          column = child_meta->col + child_meta->col_span;
+          else
+            column ++;
         }
 
       return focused;
@@ -490,8 +514,36 @@ mx_container_lower (ClutterContainer *container,
                     ClutterActor     *actor,
                     ClutterActor     *sibling)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  gint i;
+  GList *c, *position, *actor_link = NULL;
+
+  MxTablePrivate *priv = MX_TABLE (container)->priv;
+
+  if (priv->children && (priv->children->data == actor))
+    return;
+
+  position = priv->children;
+  for (c = priv->children, i = 0; c; c = c->next, i++)
+    {
+      if (c->data == actor)
+        actor_link = c;
+      if (c->data == sibling)
+        position = c;
+    }
+
+  if (!actor_link)
+    {
+      g_warning (G_STRLOC ": Actor of type '%s' is not a child of container "
+                 "of type '%s'",
+                 g_type_name (G_OBJECT_TYPE (actor)),
+                 g_type_name (G_OBJECT_TYPE (container)));
+      return;
+    }
+
+  priv->children = g_list_delete_link (priv->children, actor_link);
+  priv->children = g_list_insert_before (priv->children, position, actor);
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -499,15 +551,61 @@ mx_container_raise (ClutterContainer *container,
                     ClutterActor     *actor,
                     ClutterActor     *sibling)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  gint i;
+  GList *c, *actor_link = NULL;
+
+  gint position = -1;
+  MxTablePrivate *priv = MX_TABLE (container)->priv;
+
+  for (c = priv->children, i = 0; c; c = c->next, i++)
+    {
+      if (c->data == actor)
+        actor_link = c;
+      if (c->data == sibling)
+        position = i;
+    }
+
+  if (!actor_link)
+    {
+      g_warning (G_STRLOC ": Actor of type '%s' is not a child of container "
+                 "of type '%s'",
+                 g_type_name (G_OBJECT_TYPE (actor)),
+                 g_type_name (G_OBJECT_TYPE (container)));
+      return;
+    }
+
+  if (!actor_link->next)
+    return;
+
+  priv->children = g_list_delete_link (priv->children, actor_link);
+  priv->children = g_list_insert (priv->children, actor, position);
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
+}
+
+static gint
+mx_table_depth_sort_cb (gconstpointer a,
+                        gconstpointer b)
+{
+  gfloat depth_a = clutter_actor_get_depth ((ClutterActor *)a);
+  gfloat depth_b = clutter_actor_get_depth ((ClutterActor *)a);
+
+  if (depth_a < depth_b)
+    return -1;
+  else if (depth_a > depth_b)
+    return 1;
+  else
+    return 0;
 }
 
 static void
 mx_container_sort_depth_order (ClutterContainer *container)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  MxTablePrivate *priv = MX_TABLE (container)->priv;
+
+  priv->children = g_list_sort (priv->children, mx_table_depth_sort_cb);
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -522,6 +620,34 @@ mx_container_iface_init (ClutterContainerIface *iface)
   iface->child_meta_type = MX_TYPE_TABLE_CHILD;
 }
 
+/* MxStylable implementation */
+static void
+mx_stylable_iface_init (MxStylableIface *iface)
+{
+  static gboolean is_initialized = FALSE;
+
+  if (G_UNLIKELY (is_initialized == FALSE))
+    {
+      GParamSpec *pspec;
+
+      is_initialized = TRUE;
+
+      pspec = g_param_spec_uint ("x-mx-column-spacing",
+                                 "Column Spacing",
+                                 "The size of the column spacing",
+                                 0, G_MAXUINT, 0,
+                                 MX_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_TABLE, pspec);
+
+      pspec = g_param_spec_uint ("x-mx-row-spacing",
+                                 "Row Spacing",
+                                 "The size of the row spacing",
+                                 0, G_MAXUINT, 0,
+                                 MX_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_TABLE, pspec);
+    }
+}
+
 /* MxTable Class Implementation */
 
 static void
@@ -1609,6 +1735,26 @@ mx_table_class_init (MxTableClass *klass)
                                    pspec);
 }
 
+static void
+mx_table_style_changed (MxWidget *widget,
+                        gpointer  userdata)
+{
+  MxTable *table = MX_TABLE (widget);
+  MxTablePrivate *priv = table->priv;
+  guint row_spacing, col_spacing;
+
+  mx_stylable_get (MX_STYLABLE (widget),
+                   "x-mx-column-spacing", &col_spacing,
+                   "x-mx-row-spacing", &row_spacing,
+                   NULL);
+
+  if (!priv->ignore_css_col_spacing)
+    priv->col_spacing = col_spacing;
+
+  if (!priv->ignore_css_row_spacing)
+    priv->row_spacing = row_spacing;
+}
+
 static void
 mx_table_init (MxTable *table)
 {
@@ -1619,6 +1765,9 @@ mx_table_init (MxTable *table)
 
   table->priv->columns = g_array_new (FALSE, TRUE, sizeof (DimensionData));
   table->priv->rows = g_array_new (FALSE, TRUE, sizeof (DimensionData));
+
+  g_signal_connect (table, "style-changed",
+                    G_CALLBACK (mx_table_style_changed), NULL);
 }
 
 /* used by MxTableChild to update row/column count */
@@ -1671,6 +1820,8 @@ mx_table_set_column_spacing (MxTable *table,
     {
       priv->col_spacing = spacing;
 
+      priv->ignore_css_col_spacing = TRUE;
+
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
 
       g_object_notify (G_OBJECT (table), "column-spacing");
@@ -1699,6 +1850,8 @@ mx_table_set_row_spacing (MxTable *table,
     {
       priv->row_spacing = spacing;
 
+      priv->ignore_css_row_spacing = TRUE;
+
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
 
       g_object_notify (G_OBJECT (table), "row-spacing");
