@@ -28,7 +28,7 @@
  *
  * #MxTooltip implements a single tooltip. It should not normally be created
  * by the application but by the widget implementing tooltip capabilities, for
- * example, #mx_button_set_tooltip().
+ * example, #mx_widget_set_tooltip_text.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -67,8 +67,18 @@ struct _MxTooltipPrivate
   gboolean         actor_below;
 
   ClutterGeometry *tip_area;
+
+  CoglMatrix       stage_matrix;
+  gfloat           angle;
 };
 
+/* Time in milliseconds after a tooltip is hidden before disabling
+   browse mode */
+#define MX_TOOLTIP_BROWSE_MODE_TIMEOUT 500
+
+static gboolean mx_tooltip_in_browse_mode = FALSE;
+static guint mx_tooltip_browse_mode_timeout = 0;
+
 G_DEFINE_TYPE (MxTooltip, mx_tooltip, MX_TYPE_FLOATING_WIDGET);
 
 static void
@@ -349,9 +359,24 @@ mx_tooltip_allocate (ClutterActor          *self,
 static void
 mx_tooltip_paint (ClutterActor *self)
 {
+  gfloat width, height;
+  CoglMatrix self_matrix;
   ClutterActor *border_image, *arrow_image;
+
   MxTooltipPrivate *priv = MX_TOOLTIP (self)->priv;
 
+  clutter_actor_get_transformation_matrix (self, &self_matrix);
+  cogl_matrix_multiply (&self_matrix, &priv->stage_matrix, &self_matrix);
+  cogl_set_modelview_matrix (&self_matrix);
+
+  clutter_actor_get_size (self, &width, &height);
+  width = (gint)(width / 2.f);
+  height = (gint)(height / 2.f);
+
+  cogl_translate (width, height, 0);
+  cogl_rotate (priv->angle, 0, 0, 1);
+  cogl_translate (-width, -height, 0);
+
   border_image = mx_widget_get_border_image (MX_WIDGET (self));
   if (border_image)
     clutter_actor_paint (border_image);
@@ -363,11 +388,17 @@ mx_tooltip_paint (ClutterActor *self)
   clutter_actor_paint (priv->label);
 }
 
+static void
+mx_tooltip_stage_paint_cb (ClutterActor *stage, MxTooltip *self)
+{
+  cogl_get_modelview_matrix (&self->priv->stage_matrix);
+}
+
 static void
 mx_tooltip_map (ClutterActor *self)
 {
   MxTooltipPrivate *priv = MX_TOOLTIP (self)->priv;
-  ClutterActor *border_image, *arrow_image;
+  ClutterActor *border_image, *arrow_image, *stage;
 
   CLUTTER_ACTOR_CLASS (mx_tooltip_parent_class)->map (self);
 
@@ -380,13 +411,20 @@ mx_tooltip_map (ClutterActor *self)
     clutter_actor_map (arrow_image);
 
   clutter_actor_map (priv->label);
+
+  stage = clutter_actor_get_stage (self);
+  g_signal_connect (stage, "paint",
+                    G_CALLBACK (mx_tooltip_stage_paint_cb), self);
 }
 
 static void
 mx_tooltip_unmap (ClutterActor *self)
 {
   MxTooltipPrivate *priv = MX_TOOLTIP (self)->priv;
-  ClutterActor *border_image, *arrow_image;
+  ClutterActor *border_image, *arrow_image, *stage;
+
+  stage = clutter_actor_get_stage (self);
+  g_signal_handlers_disconnect_by_func (stage, mx_tooltip_stage_paint_cb, self);
 
   CLUTTER_ACTOR_CLASS (mx_tooltip_parent_class)->unmap (self);
 
@@ -455,7 +493,8 @@ mx_tooltip_class_init (MxTooltipClass *klass)
   pspec = g_param_spec_string ("text",
                                "Text",
                                "Text of the tooltip",
-                               NULL, G_PARAM_READWRITE);
+                               NULL,
+                               G_PARAM_READWRITE | MX_PARAM_TRANSLATEABLE);
   g_object_class_install_property (gobject_class, PROP_TEXT, pspec);
 
   pspec = g_param_spec_boxed ("tip-area",
@@ -495,10 +534,19 @@ mx_tooltip_update_position (MxTooltip *tooltip)
 {
   MxTooltipPrivate *priv = tooltip->priv;
   ClutterGeometry *tip_area = tooltip->priv->tip_area;
-  gfloat tooltip_w, tooltip_h, tooltip_x, tooltip_y;
-  gfloat transformed_x, transformed_y;
+  gfloat tooltip_w, tooltip_h, tooltip_x, tooltip_y, abs_x, abs_y;
+  ClutterActor *stage, *parent;
+  MxWindowRotation rotation;
   gfloat stage_w, stage_h;
-  ClutterActor *stage;
+  MxWindow *window;
+
+  /* If there's no stage, bail out - there's nothing we can do */
+  stage = clutter_actor_get_stage ((ClutterActor *) tooltip);
+  if (!stage)
+    return;
+
+  parent = clutter_actor_get_parent ((ClutterActor *) tooltip);
+  clutter_actor_get_transformed_position (parent, &abs_x, &abs_y);
 
   /* ensure the tooltip with is not fixed size */
   clutter_actor_set_size ((ClutterActor*) tooltip, -1, -1);
@@ -506,10 +554,20 @@ mx_tooltip_update_position (MxTooltip *tooltip)
   /* if no area set, just position ourselves top left */
   if (!priv->tip_area)
     {
-      clutter_actor_set_position ((ClutterActor*) tooltip, 0, 0);
+      clutter_actor_set_position ((ClutterActor*) tooltip, abs_x, abs_y);
       return;
     }
 
+  /* check if we're in a window and if there's rotation */
+  window = mx_window_get_for_stage (CLUTTER_STAGE (stage));
+  if (window)
+    g_object_get (G_OBJECT (window),
+                  "window-rotation", &rotation,
+                  "window-rotation-angle", &priv->angle,
+                  NULL);
+  else
+    rotation = MX_WINDOW_ROTATION_0;
+
   /* we need to have a style in case there are padding values to take into
    * account when calculating width/height */
   mx_stylable_style_changed (MX_STYLABLE (tooltip), MX_STYLE_CHANGED_FORCE);
@@ -517,59 +575,111 @@ mx_tooltip_update_position (MxTooltip *tooltip)
   /* find out the tooltip's size */
   clutter_actor_get_size ((ClutterActor*) tooltip, &tooltip_w, &tooltip_h);
 
-  /* attempt to place the tooltip */
-  tooltip_x = (int)(tip_area->x + (tip_area->width / 2) - (tooltip_w / 2));
-  tooltip_y = (int)(tip_area->y + tip_area->height);
-
-  stage = clutter_actor_get_stage ((ClutterActor *) tooltip);
-  if (!stage)
-    {
-      return;
-    }
+  /* find out the stage's size to keep the tooltip on-screen */
   clutter_actor_get_size (stage, &stage_w, &stage_h);
 
-  /* make sure the tooltip is not off screen vertically */
-  if (tooltip_w > stage_w)
+  /* attempt to place the tooltip */
+  /* This special-cases the 4 window rotations, as doing this with
+   * arbitrary rotations would massively complicate the code for
+   * little benefit.
+   */
+  priv->actor_below = FALSE;
+  switch (rotation)
     {
-      tooltip_x = 0;
-      clutter_actor_set_width ((ClutterActor*) tooltip, stage_w);
-    }
-  else if (tooltip_x < 0)
-    {
-      tooltip_x = 0;
-    }
-  else if (tooltip_x + tooltip_w > stage_w)
-    {
-      tooltip_x = (int)(stage_w) - tooltip_w;
+    case MX_WINDOW_ROTATION_0:
+      tooltip_x = (int)(tip_area->x + (tip_area->width / 2) -
+                        (tooltip_w / 2));
+      tooltip_y = (int)(tip_area->y + tip_area->height);
+
+      /* Keep on the screen vertically */
+      if (tooltip_y + tooltip_h > stage_h)
+        {
+          priv->actor_below = TRUE;
+
+          /* re-query size as may have changed */
+          clutter_actor_get_preferred_height ((ClutterActor*) tooltip,
+                                              -1, NULL, &tooltip_h);
+          tooltip_y = tip_area->y - tooltip_h;
+        }
+      break;
+
+    case MX_WINDOW_ROTATION_180:
+      tooltip_x = (int)(tip_area->x + (tip_area->width / 2) -
+                        (tooltip_w / 2));
+      tooltip_y = (int)(tip_area->y - tooltip_h);
+
+      /* Keep on the screen vertically */
+      if (tooltip_y < 0)
+        {
+          priv->actor_below = TRUE;
+          tooltip_y = tip_area->y + tip_area->height;
+        }
+      break;
+
+    case MX_WINDOW_ROTATION_90:
+      tooltip_x = (int)(tip_area->x - (tooltip_w / 2) - (tooltip_h / 2));
+      tooltip_y = (int)(tip_area->y + (tip_area->height / 2) -
+                        (tooltip_h / 2));
+
+      /* Keep on the screen vertically */
+      if (tooltip_x + tooltip_w / 2 - tooltip_h / 2 < 0)
+        {
+          priv->actor_below = TRUE;
+          tooltip_x = tip_area->x + tip_area->width + tooltip_h / 2 -
+                      tooltip_w / 2;
+        }
+      break;
+
+    case MX_WINDOW_ROTATION_270:
+      tooltip_x = (int)(tip_area->x + tip_area->width -
+                        (tooltip_w / 2) + (tooltip_h / 2));
+      tooltip_y = (int)(tip_area->y + (tip_area->height / 2) -
+                        (tooltip_h / 2));
+
+      /* Keep on the screen vertically */
+      if (tooltip_x + tooltip_w / 2 + tooltip_h / 2 > stage_w)
+        {
+          priv->actor_below = TRUE;
+          tooltip_x = tip_area->x - tooltip_h / 2 - tooltip_w / 2;
+        }
+      break;
     }
 
-  /* make sure the tooltip is not off screen horizontally */
-  if (tooltip_y + tooltip_h > stage_h)
+  /* Keep on the screen horizontally */
+  switch (rotation)
     {
-      priv->actor_below = TRUE;
+    case MX_WINDOW_ROTATION_0:
+    case MX_WINDOW_ROTATION_180:
+      if (tooltip_w > stage_w)
+        {
+          tooltip_x = 0;
+          clutter_actor_set_width ((ClutterActor*) tooltip, stage_w);
+        }
+      else if (tooltip_x < 0)
+        tooltip_x = 0;
+      else if (tooltip_x + tooltip_w > stage_w)
+        tooltip_x = (int)(stage_w) - tooltip_w;
+      break;
 
-      /* re-query size as may have changed */
-      clutter_actor_get_preferred_height ((ClutterActor*) tooltip,
-                                          -1, NULL, &tooltip_h);
-      tooltip_y = tip_area->y - tooltip_h;
-    }
-  else
-    {
-      priv->actor_below = FALSE;
+    case MX_WINDOW_ROTATION_90:
+    case MX_WINDOW_ROTATION_270:
+      if (tooltip_w > stage_h)
+        {
+          tooltip_y = stage_h / 2 - tooltip_h / 2;
+          tooltip_x += (tooltip_w - stage_h) / 2;
+
+          clutter_actor_set_width ((ClutterActor*) tooltip, stage_h);
+        }
+      else if (tooltip_y - tooltip_w / 2 + tooltip_h / 2 < 0)
+        tooltip_y = tooltip_w / 2 - tooltip_h / 2;
+      else if (tooltip_y + tooltip_w / 2 + tooltip_h / 2 > stage_h)
+        tooltip_y = (int)(stage_h - tooltip_w / 2 - tooltip_h / 2);
+      break;
     }
 
   /* calculate the arrow offset */
   priv->arrow_offset = tip_area->x + tip_area->width / 2 - tooltip_x;
-
-  /* co-ordinates were calculated with respect to the stage, so now
-   * convert them to be relative to the parent */
-
-  clutter_actor_get_transformed_position (clutter_actor_get_parent (CLUTTER_ACTOR (tooltip)),
-                                          &transformed_x, &transformed_y);
-
-  clutter_actor_set_position ((ClutterActor*) tooltip,
-                              tooltip_x - transformed_x,
-                              tooltip_y - transformed_y);
+  clutter_actor_set_position ((ClutterActor*) tooltip, tooltip_x, tooltip_y);
 }
 
 /**
@@ -649,6 +759,23 @@ mx_tooltip_show (MxTooltip *tooltip)
                          "scale-x", 1.0,
                          "scale-y", 1.0,
                          NULL);
+
+  /* Enter browse mode */
+  mx_tooltip_in_browse_mode = TRUE;
+  /* Disable any previous queued attempts to leave browse mode */
+  if (mx_tooltip_browse_mode_timeout)
+    {
+      g_source_remove (mx_tooltip_browse_mode_timeout);
+      mx_tooltip_browse_mode_timeout = 0;
+    }
+}
+
+static gboolean
+mx_tooltip_browse_mode_timeout_cb (gpointer data)
+{
+  mx_tooltip_in_browse_mode = FALSE;
+  mx_tooltip_browse_mode_timeout = 0;
+  return FALSE;
 }
 
 static void
@@ -690,6 +817,14 @@ mx_tooltip_hide (MxTooltip *tooltip)
                            NULL);
   g_signal_connect (animation, "completed",
                     G_CALLBACK (mx_tooltip_hide_complete), tooltip);
+
+  /* Leave browse mode after a short delay */
+  if (mx_tooltip_browse_mode_timeout)
+    g_source_remove (mx_tooltip_browse_mode_timeout);
+  mx_tooltip_browse_mode_timeout =
+    g_timeout_add (MX_TOOLTIP_BROWSE_MODE_TIMEOUT,
+                   mx_tooltip_browse_mode_timeout_cb,
+                   NULL);
 }
 
 /**
@@ -726,3 +861,20 @@ mx_tooltip_get_tip_area (MxTooltip *tooltip)
 
   return tooltip->priv->tip_area;
 }
+
+/**
+ * mx_tooltip_is_in_browse_mode:
+ *
+ * Browse mode is entered whenever a tooltip is displayed and it is
+ * left after a short delay when a tooltip is hidden. This is used to
+ * make tooltips display quicker when a previous tooltip is already
+ * displayed.
+ *
+ * Returns: %TRUE if the app is in tooltip browse mode or %FALSE
+ * otherwise.
+ */
+gboolean
+mx_tooltip_is_in_browse_mode (void)
+{
+  return mx_tooltip_in_browse_mode;
+}
