@@ -30,6 +30,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <libintl.h>
 
 #include <clutter/clutter.h>
 
@@ -42,8 +43,7 @@
 #include "mx-texture-frame.h"
 #include "mx-tooltip.h"
 #include "mx-enum-types.h"
-
-#define LONG_PRESS_TIMEOUT 500
+#include "mx-settings.h"
 
 /*
  * Forward declaration for sake of MxWidgetChild
@@ -65,11 +65,14 @@ struct _MxWidgetPrivate
 
   guint         is_hovered : 1;
   guint         is_disabled : 1;
+  guint         parent_disabled : 1;
 
   MxTooltip    *tooltip;
   MxMenu       *menu;
 
   guint         long_press_source;
+
+  guint         tooltip_timeout;
 };
 
 /**
@@ -107,11 +110,18 @@ enum
 static guint widget_signals[LAST_SIGNAL] = { 0, };
 
 static void mx_stylable_iface_init (MxStylableIface *iface);
+static ClutterScriptableIface *parent_scriptable_iface = NULL;
+static void scriptable_iface_init (ClutterScriptableIface *iface);
 
+/* Length of time in milliseconds that the cursor must be held steady
+   over a widget before the tooltip is displayed */
+#define MX_WIDGET_TOOLTIP_TIMEOUT 500
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (MxWidget, mx_widget, CLUTTER_TYPE_ACTOR,
                                   G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
-                                                         mx_stylable_iface_init));
+                                                         mx_stylable_iface_init)
+                                  G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                         scriptable_iface_init));
 
 #define MX_WIDGET_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), MX_TYPE_WIDGET, MxWidgetPrivate))
 
@@ -197,6 +207,45 @@ mx_widget_get_property (GObject    *gobject,
     }
 }
 
+static gboolean
+mx_widget_tooltip_timeout_cb (gpointer data)
+{
+  MxWidget *self = MX_WIDGET (data);
+  MxWidgetPrivate *priv = self->priv;
+
+  mx_widget_show_tooltip (self);
+
+  priv->tooltip_timeout = 0;
+
+  return FALSE;
+}
+
+static void
+mx_widget_remove_tooltip_timeout (MxWidget *widget)
+{
+  MxWidgetPrivate *priv = widget->priv;
+
+  if (priv->tooltip_timeout)
+    {
+      g_source_remove (priv->tooltip_timeout);
+      priv->tooltip_timeout = 0;
+    }
+}
+
+static void
+mx_widget_set_tooltip_timeout (MxWidget *widget)
+{
+  MxWidgetPrivate *priv = widget->priv;
+
+  /* Remove any existing tooltip timeout so that we can start again */
+  mx_widget_remove_tooltip_timeout (widget);
+
+  priv->tooltip_timeout =
+    clutter_threads_add_timeout (MX_WIDGET_TOOLTIP_TIMEOUT,
+                                 mx_widget_tooltip_timeout_cb,
+                                 widget);
+}
+
 static void
 mx_widget_dispose (GObject *gobject)
 {
@@ -249,6 +298,8 @@ mx_widget_finalize (GObject *gobject)
 {
   MxWidgetPrivate *priv = MX_WIDGET (gobject)->priv;
 
+  mx_widget_remove_tooltip_timeout (MX_WIDGET (gobject));
+
   g_free (priv->style_class);
   g_free (priv->pseudo_class);
 
@@ -270,8 +321,6 @@ mx_widget_allocate (ClutterActor          *actor,
 {
   MxWidgetPrivate *priv = MX_WIDGET (actor)->priv;
   ClutterActorClass *klass;
-  ClutterGeometry area;
-  ClutterVertex in_v, out_v;
 
   klass = CLUTTER_ACTOR_CLASS (mx_widget_parent_class);
   klass->allocate (actor, box, flags);
@@ -279,16 +328,31 @@ mx_widget_allocate (ClutterActor          *actor,
   /* update tooltip position */
   if (priv->tooltip)
     {
-      in_v.x = in_v.y = in_v.z = 0;
-      clutter_actor_apply_transform_to_point (actor, &in_v, &out_v);
-      area.x = out_v.x;
-      area.y = out_v.y;
+      ClutterVertex verts[4];
+      ClutterGeometry area;
+      gfloat x, y, x2, y2;
+      gint i;
 
-      in_v.x = box->x2 - box->x1;
-      in_v.y = box->y2 - box->y1;
-      clutter_actor_apply_transform_to_point (actor, &in_v, &out_v);
-      area.width = out_v.x - area.x;
-      area.height = out_v.y - area.y;
+      clutter_actor_get_abs_allocation_vertices (actor, verts);
+
+      x = y = G_MAXFLOAT;
+      x2 = y2 = -G_MAXFLOAT;
+      for (i = 0; i < G_N_ELEMENTS (verts); i++)
+        {
+          if (verts[i].x < x)
+            x = verts[i].x;
+          if (verts[i].x > x2)
+            x2 = verts[i].x;
+          if (verts[i].y < y)
+            y = verts[i].y;
+          if (verts[i].y > y2)
+            y2 = verts[i].y;
+        }
+
+      area.x = x;
+      area.y = y;
+      area.width = x2 - x;
+      area.height = y2 - y;
 
       mx_tooltip_set_tip_area (priv->tooltip, &area);
     }
@@ -727,14 +791,8 @@ mx_widget_enter (ClutterActor         *actor,
   MxWidget *widget = MX_WIDGET (actor);
   MxWidgetPrivate *priv = widget->priv;
 
-  if (event->source != actor)
-    return FALSE;
-
-  if (priv->tooltip)
-    mx_widget_show_tooltip (widget);
-
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
-      return TRUE;
+  /* This is also expected to handle enter events from child actors
+     because they will bubble up */
 
   mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
   priv->is_hovered = TRUE;
@@ -749,14 +807,14 @@ mx_widget_leave (ClutterActor         *actor,
   MxWidget *widget = MX_WIDGET (actor);
   MxWidgetPrivate *priv = widget->priv;
 
-  if (event->source != actor)
+  /* If the leave event is simply moving to a child actor then we'll
+     ignore it so that the actor will retain its hover state until the
+     pointer moves to an unrelated actor. For this to work the actual
+     leave event from the child needs to bubble up to this actor */
+  if (clutter_actor_contains (actor, event->related))
     return FALSE;
 
-  if (priv->tooltip)
-    mx_tooltip_hide (priv->tooltip);
-
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
-      return TRUE;
+  mx_widget_hide_tooltip (widget);
 
   mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "");
   priv->is_hovered = FALSE;
@@ -764,6 +822,25 @@ mx_widget_leave (ClutterActor         *actor,
   return FALSE;
 }
 
+static gboolean
+mx_widget_motion (ClutterActor       *actor,
+                  ClutterMotionEvent *event)
+{
+  MxWidget *widget = MX_WIDGET (actor);
+  MxWidgetPrivate *priv = widget->priv;
+
+  if (priv->tooltip && !CLUTTER_ACTOR_IS_VISIBLE (priv->tooltip))
+    {
+      /* If tooltips are in browse mode then display the tooltip immediately */
+      if (mx_tooltip_is_in_browse_mode ())
+        mx_widget_show_tooltip (widget);
+      else
+        mx_widget_set_tooltip_timeout (widget);
+    }
+
+  return FALSE;
+}
+
 static gboolean
 mx_widget_emit_long_press (MxWidget *widget)
 {
@@ -789,12 +866,16 @@ mx_widget_long_press_query (MxWidget           *widget,
 {
   MxWidgetPrivate *priv = widget->priv;
   gboolean query_result = FALSE;
+  MxSettings *settings = mx_settings_get_default ();
+  guint timeout;
+
+  g_object_get (settings, "long-press-timeout", &timeout, NULL);
 
   g_signal_emit (widget, widget_signals[LONG_PRESS], 0, event->x,
                  event->y, MX_LONG_PRESS_QUERY, &query_result);
 
   if (query_result)
-    priv->long_press_source = g_timeout_add (LONG_PRESS_TIMEOUT,
+    priv->long_press_source = g_timeout_add (timeout,
                                              (GSourceFunc) mx_widget_emit_long_press,
                                              widget);
 }
@@ -822,6 +903,31 @@ mx_widget_long_press_cancel (MxWidget *widget)
     }
 }
 
+/**
+ * mx_widget_apply_style:
+ * @widget: A #MxWidget
+ * @style: A #MxStyle
+ *
+ * Used to implement how a new style instance should be applied in the widget.
+ * For instance, setting style instance on stylable internal children.
+ *
+ * Since: 1.2
+ */
+void
+mx_widget_apply_style (MxWidget *widget,
+                       MxStyle  *style)
+{
+  MxWidgetClass *klass;
+
+  g_return_if_fail (MX_IS_WIDGET (widget));
+  g_return_if_fail (style != NULL);
+
+  klass = MX_WIDGET_GET_CLASS (widget);
+
+  if (klass->apply_style != NULL)
+    klass->apply_style (widget, style);
+}
+
 static gboolean
 mx_widget_button_press (ClutterActor       *actor,
                         ClutterButtonEvent *event)
@@ -867,12 +973,77 @@ mx_widget_hide (ClutterActor *actor)
   MxWidget *widget = (MxWidget *) actor;
 
   /* hide the tooltip, if there is one */
-  if (widget->priv->tooltip)
-    mx_tooltip_hide (widget->priv->tooltip);
+  mx_widget_hide_tooltip (widget);
 
   CLUTTER_ACTOR_CLASS (mx_widget_parent_class)->hide (actor);
 }
 
+static void
+_mx_widget_propagate_disabled (ClutterContainer *container,
+                               gboolean          disabled)
+{
+  GList *c, *children;
+
+  children = clutter_container_get_children (container);
+
+  /* Recurse through the children and set the 'parent_disabled' flag. */
+  for (c = children; c; c = c->next)
+    {
+      MxWidget *child = c->data;
+
+      if (MX_IS_WIDGET (child))
+        {
+          MxWidgetPrivate *child_priv = child->priv;
+
+          child_priv->parent_disabled = disabled;
+
+          /* If this child has already been disabled explicitly,
+           * we don't need to recurse through its children to set
+           * the 'parent_disabled' flag, as it'll already be set.
+           */
+          if (child_priv->is_disabled)
+            continue;
+        }
+
+      if (CLUTTER_IS_CONTAINER (child))
+        _mx_widget_propagate_disabled ((ClutterContainer *) child, disabled);
+    }
+}
+
+static void
+mx_widget_parent_set (ClutterActor *actor,
+                      ClutterActor *old_parent)
+{
+  ClutterActor *parent;
+  MxWidget *widget = (MxWidget *) actor;
+  MxWidgetPrivate *priv = widget->priv;
+
+  /* Make sure the disabled state cache remains valid */
+  if (!priv->is_disabled)
+    {
+      gboolean disabled;
+
+      parent = clutter_actor_get_parent (actor);
+      while (parent && !MX_IS_WIDGET (parent))
+        parent = clutter_actor_get_parent (parent);
+
+      if (parent)
+        {
+          MxWidgetPrivate *parent_priv = ((MxWidget *) parent)->priv;
+          disabled = parent_priv->is_disabled || parent_priv->parent_disabled;
+        }
+      else
+        disabled = FALSE;
+
+      if (disabled != priv->parent_disabled)
+        {
+          priv->parent_disabled = disabled;
+          if (CLUTTER_IS_CONTAINER (widget))
+            _mx_widget_propagate_disabled ((ClutterContainer *) widget,
+                                           disabled);
+        }
+    }
+}
 
 
 static void
@@ -897,10 +1068,12 @@ mx_widget_class_init (MxWidgetClass *klass)
 
   actor_class->enter_event = mx_widget_enter;
   actor_class->leave_event = mx_widget_leave;
+  actor_class->motion_event = mx_widget_motion;
   actor_class->button_press_event = mx_widget_button_press;
   actor_class->button_release_event = mx_widget_button_release;
 
   actor_class->hide = mx_widget_hide;
+  actor_class->parent_set = mx_widget_parent_set;
 
   klass->paint_background = mx_widget_real_paint_background;
 
@@ -920,7 +1093,7 @@ mx_widget_class_init (MxWidgetClass *klass)
                                "Tooltip Text",
                                "Text displayed on the tooltip",
                                "",
-                               MX_PARAM_READWRITE);
+                               MX_PARAM_READWRITE | MX_PARAM_TRANSLATEABLE);
   g_object_class_install_property (gobject_class, PROP_TOOLTIP_TEXT, pspec);
 
   /**
@@ -983,6 +1156,8 @@ mx_widget_set_style (MxStylable *stylable,
     g_object_unref (priv->style);
 
   priv->style = g_object_ref_sink (style);
+
+  mx_widget_apply_style (MX_WIDGET (stylable), style);
 }
 
 static void
@@ -1155,7 +1330,7 @@ mx_widget_init (MxWidget *actor)
   actor->priv = priv = MX_WIDGET_GET_PRIVATE (actor);
 
   /* set the default style */
-  mx_widget_set_style (MX_STYLABLE (actor), mx_style_get_default ());
+  mx_stylable_set_style (MX_STYLABLE (actor), mx_style_get_default ());
 
   /* connect the notifiers for the stylable */
   mx_stylable_connect_change_notifiers (MX_STYLABLE (actor));
@@ -1245,6 +1420,8 @@ mx_widget_set_has_tooltip (MxWidget *widget,
           clutter_actor_unparent (CLUTTER_ACTOR (priv->tooltip));
           priv->tooltip = NULL;
         }
+
+      mx_widget_remove_tooltip_timeout (widget);
     }
 }
 
@@ -1323,22 +1500,40 @@ mx_widget_get_tooltip_text (MxWidget *widget)
 void
 mx_widget_show_tooltip (MxWidget *widget)
 {
-  gfloat x, y, width, height;
+  gint i;
+  gfloat x, y, x2, y2;
   ClutterGeometry area;
+  ClutterVertex verts[4];
 
   g_return_if_fail (MX_IS_WIDGET (widget));
 
+  /* Remove any timeout so we don't show the tooltip again */
+  mx_widget_remove_tooltip_timeout (widget);
+
   /* XXX not necceary, but first allocate transform is wrong */
 
-  clutter_actor_get_transformed_position ((ClutterActor*) widget,
-                                          &x, &y);
+  /* Work out the bounding box */
+  clutter_actor_get_abs_allocation_vertices ((ClutterActor*) widget,
+                                             verts);
 
-  clutter_actor_get_size ((ClutterActor*) widget, &width, &height);
+  x = y = G_MAXFLOAT;
+  x2 = y2 = -G_MAXFLOAT;
+  for (i = 0; i < G_N_ELEMENTS (verts); i++)
+    {
+      if (verts[i].x < x)
+        x = verts[i].x;
+      if (verts[i].x > x2)
+        x2 = verts[i].x;
+      if (verts[i].y < y)
+        y = verts[i].y;
+      if (verts[i].y > y2)
+        y2 = verts[i].y;
+    }
 
   area.x = x;
   area.y = y;
-  area.width = width;
-  area.height = height;
+  area.width = x2 - x;
+  area.height = y2 - y;
 
 
   if (widget->priv->tooltip)
@@ -1360,6 +1555,8 @@ mx_widget_hide_tooltip (MxWidget *widget)
 {
   g_return_if_fail (MX_IS_WIDGET (widget));
 
+  mx_widget_remove_tooltip_timeout (widget);
+
   if (widget->priv->tooltip)
     mx_tooltip_hide (widget->priv->tooltip);
 }
@@ -1481,6 +1678,10 @@ mx_widget_set_disabled (MxWidget *widget,
     {
       priv->is_disabled = disabled;
 
+      /* Propagate the disabled state to our children, if necessary */
+      if (!priv->parent_disabled && CLUTTER_IS_CONTAINER (widget))
+        _mx_widget_propagate_disabled ((ClutterContainer *) widget, disabled);
+
       /* when a widget is disabled, get_style_pseudo_class will always return
        * "disabled" */
 
@@ -1501,23 +1702,45 @@ mx_widget_set_disabled (MxWidget *widget,
 gboolean
 mx_widget_get_disabled (MxWidget *widget)
 {
-  ClutterActor *parent;
-
   g_return_val_if_fail (MX_IS_WIDGET (widget), FALSE);
+  return widget->priv->is_disabled || widget->priv->parent_disabled;
+}
 
-  if (widget->priv->is_disabled)
-    return TRUE;
+/* Support translateable strings from JSON */
+static void
+widget_scriptable_set_custom_property (ClutterScriptable *scriptable,
+                                       ClutterScript     *script,
+                                       const gchar       *name,
+                                       const GValue      *value)
+{
+  GParamSpec *pspec;
 
-  /* check if any of the parents are disabled */
-  parent = clutter_actor_get_parent (CLUTTER_ACTOR (widget));
+  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (scriptable), name);
 
-  while (parent)
+  if (pspec->flags & MX_PARAM_TRANSLATEABLE &&
+      pspec->value_type == G_TYPE_STRING)
     {
-      if (MX_IS_WIDGET (parent) && MX_WIDGET (parent)->priv->is_disabled)
-        return TRUE;
+      g_object_set (scriptable, name,
+                    gettext (g_value_get_string (value)), NULL);
+    }
+  else
+    {
+      /* chain up */
+      if (parent_scriptable_iface->set_custom_property)
+        parent_scriptable_iface->set_custom_property (scriptable, script,
+                                                      name,
+                                                      value);
+    }
+}
 
-      parent = clutter_actor_get_parent (CLUTTER_ACTOR (parent));
-    }
+static void
+scriptable_iface_init (ClutterScriptableIface *iface)
+{
+  parent_scriptable_iface = g_type_interface_peek_parent (iface);
+
+  if (!parent_scriptable_iface)
+    parent_scriptable_iface = g_type_default_interface_peek
+                                          (CLUTTER_TYPE_SCRIPTABLE);
 
-  return FALSE;
+  iface->set_custom_property = widget_scriptable_set_custom_property;
 }
