@@ -38,6 +38,7 @@
 #include <mx/mx-deform-bow-tie.h>
 #include <mx/mx-deform-page-turn.h>
 #include <mx/mx-deform-waves.h>
+#include <mx/mx-dialog.h>
 #include <mx/mx-draggable.h>
 #include <mx/mx-droppable.h>
 #include <mx/mx-clipboard.h>
@@ -48,9 +49,11 @@
 #include <mx/mx-focus-manager.h>
 #include <mx/mx-focusable.h>
 #include <mx/mx-frame.h>
+#include <mx/mx-kinetic-scroll-view.h>
 #include <mx/mx-grid.h>
-#include <mx/mx-icon.h>
 #include <mx/mx-icon-theme.h>
+#include <mx/mx-icon.h>
+#include <mx/mx-image.h>
 #include <mx/mx-item-factory.h>
 #include <mx/mx-item-view.h>
 #include <mx/mx-list-view.h>
@@ -63,7 +66,11 @@
 #include <mx/mx-scroll-bar.h>
 #include <mx/mx-scroll-view.h>
 #include <mx/mx-scrollable.h>
+#include <mx/mx-settings.h>
 #include <mx/mx-slider.h>
+#include <mx/mx-spinner.h>
+#include <mx/mx-stack.h>
+#include <mx/mx-stack-child.h>
 #include <mx/mx-stylable.h>
 #include <mx/mx-style.h>
 #include <mx/mx-table.h>
