@@ -24,6 +24,129 @@
 #include "test-mx.h"
 
 
+static void
+rotate_left_clicked_cb (ClutterActor *button,
+                        MxWindow     *window)
+{
+  MxWindowRotation rotation = mx_window_get_window_rotation (window);
+
+  switch (rotation)
+    {
+    case MX_WINDOW_ROTATION_0:
+      rotation = MX_WINDOW_ROTATION_270;
+      break;
+    case MX_WINDOW_ROTATION_90:
+      rotation = MX_WINDOW_ROTATION_0;
+      break;
+    case MX_WINDOW_ROTATION_180:
+      rotation = MX_WINDOW_ROTATION_90;
+      break;
+    case MX_WINDOW_ROTATION_270:
+      rotation = MX_WINDOW_ROTATION_180;
+      break;
+    }
+
+  mx_window_set_window_rotation (window, rotation);
+}
+
+static void
+rotate_right_clicked_cb (ClutterActor *button,
+                         MxWindow     *window)
+{
+  MxWindowRotation rotation = mx_window_get_window_rotation (window);
+
+  switch (rotation)
+    {
+    case MX_WINDOW_ROTATION_0:
+      rotation = MX_WINDOW_ROTATION_90;
+      break;
+    case MX_WINDOW_ROTATION_90:
+      rotation = MX_WINDOW_ROTATION_180;
+      break;
+    case MX_WINDOW_ROTATION_180:
+      rotation = MX_WINDOW_ROTATION_270;
+      break;
+    case MX_WINDOW_ROTATION_270:
+      rotation = MX_WINDOW_ROTATION_0;
+      break;
+    }
+
+  mx_window_set_window_rotation (window, rotation);
+}
+
+static void
+rotate_180_clicked_cb (ClutterActor *button,
+                       MxWindow     *window)
+{
+  MxWindowRotation rotation = mx_window_get_window_rotation (window);
+
+  switch (rotation)
+    {
+    case MX_WINDOW_ROTATION_0:
+      rotation = MX_WINDOW_ROTATION_180;
+      break;
+    case MX_WINDOW_ROTATION_90:
+      rotation = MX_WINDOW_ROTATION_270;
+      break;
+    case MX_WINDOW_ROTATION_180:
+      rotation = MX_WINDOW_ROTATION_0;
+      break;
+    case MX_WINDOW_ROTATION_270:
+      rotation = MX_WINDOW_ROTATION_90;
+      break;
+    }
+
+  mx_window_set_window_rotation (window, rotation);
+}
+
+static ClutterActor *
+create_rotate_box (MxWindow *window)
+{
+  ClutterActor *button, *icon, *icon2, *layout2, *spinner;
+  ClutterActor *layout = mx_box_layout_new ();
+
+  /* Create rotate-left button */
+  icon = mx_icon_new ();
+  mx_icon_set_icon_name (MX_ICON (icon), "object-rotate-left");
+  mx_icon_set_icon_size (MX_ICON (icon), 16);
+  button = mx_button_new ();
+  mx_bin_set_child (MX_BIN (button), icon);
+  g_signal_connect (button, "clicked",
+                    G_CALLBACK (rotate_left_clicked_cb), window);
+  clutter_container_add_actor (CLUTTER_CONTAINER (layout), button);
+
+  /* Create rotate-180 button */
+  icon = mx_icon_new ();
+  mx_icon_set_icon_name (MX_ICON (icon), "object-rotate-left");
+  mx_icon_set_icon_size (MX_ICON (icon), 16);
+  icon2 = mx_icon_new ();
+  mx_icon_set_icon_name (MX_ICON (icon2), "object-rotate-right");
+  mx_icon_set_icon_size (MX_ICON (icon2), 16);
+  layout2 = mx_box_layout_new ();
+  clutter_container_add (CLUTTER_CONTAINER (layout2), icon, icon2, NULL);
+  button = mx_button_new ();
+  mx_bin_set_child (MX_BIN (button), layout2);
+  g_signal_connect (button, "clicked",
+                    G_CALLBACK (rotate_180_clicked_cb), window);
+  clutter_container_add_actor (CLUTTER_CONTAINER (layout), button);
+
+  /* Create rotate-right button */
+  icon = mx_icon_new ();
+  mx_icon_set_icon_name (MX_ICON (icon), "object-rotate-right");
+  mx_icon_set_icon_size (MX_ICON (icon), 16);
+  button = mx_button_new ();
+  mx_bin_set_child (MX_BIN (button), icon);
+  g_signal_connect (button, "clicked",
+                    G_CALLBACK (rotate_right_clicked_cb), window);
+  clutter_container_add_actor (CLUTTER_CONTAINER (layout), button);
+
+  /* Create spinner */
+  spinner = mx_spinner_new ();
+  clutter_container_add_actor (CLUTTER_CONTAINER (layout), spinner);
+
+  return layout;
+}
+
 static void
 show_page (MxButton         *button,
            GParamSpec       *pspec,
@@ -133,7 +256,8 @@ add_tab (ClutterContainer *box,
 int
 main (int argc, char *argv[])
 {
-  ClutterActor *stage, *vbox, *hbox, *holder, *mainbox, *toolbar, *combo;
+  ClutterActor *stage, *vbox, *hbox, *holder, *mainbox, *toolbar, *combo,
+               *layout;
   MxAction *prev, *next;
   MxButtonGroup *group;
   MxApplication *application;
@@ -152,15 +276,27 @@ main (int argc, char *argv[])
 
   /* create the toolbar */
   toolbar = (ClutterActor *)mx_window_get_toolbar (window);
-  mx_bin_set_alignment (MX_BIN (toolbar), MX_ALIGN_MIDDLE, MX_ALIGN_MIDDLE);
+  mx_bin_set_fill (MX_BIN (toolbar), TRUE, FALSE);
+
+  layout = mx_box_layout_new ();
 
   combo = mx_combo_box_new ();
-  clutter_container_add_actor (CLUTTER_CONTAINER (toolbar), combo);
+  clutter_container_add_actor (CLUTTER_CONTAINER (layout), combo);
   mx_combo_box_set_active_text (MX_COMBO_BOX (combo), "Select");
   mx_combo_box_set_active_icon_name (MX_COMBO_BOX (combo), "dialog-question");
   mx_combo_box_append_text (MX_COMBO_BOX (combo), "Hello");
   mx_combo_box_append_text (MX_COMBO_BOX (combo), "World");
 
+  mx_box_layout_add_actor_with_properties (MX_BOX_LAYOUT (layout),
+                                           create_rotate_box (window),
+                                           1,
+                                           "x-fill", FALSE,
+                                           "x-align", MX_ALIGN_END,
+                                           "expand", TRUE,
+                                           NULL);
+
+  clutter_container_add_actor (CLUTTER_CONTAINER (toolbar), layout);
+
   /* create the horizontal layout */
   hbox = mx_box_layout_new ();
 
@@ -207,6 +343,8 @@ main (int argc, char *argv[])
            (GCallback) scroll_view_main, CLUTTER_CONTAINER (holder));
   add_tab (CLUTTER_CONTAINER (vbox), group, "Styles",
            (GCallback) styles_main, CLUTTER_CONTAINER (holder));
+  add_tab (CLUTTER_CONTAINER (vbox), group, "Dialog",
+           (GCallback) dialog_main, CLUTTER_CONTAINER (holder));
 
   prev = mx_action_new_full ("Previous tab",
                              "Previous tab",
