@@ -19,10 +19,13 @@
 #include <clutter/clutter.h>
 #include <mx/mx.h>
 
+typedef void (*TestMxCallback)(ClutterContainer *container);
+
 void label_main (ClutterContainer *container);
 void entry_main (ClutterContainer *container);
 void tooltips_main (ClutterContainer *container);
 void buttons_main (ClutterContainer *container);
+void button_group_main (ClutterContainer *container);
 void combo_box_main (ClutterContainer *container);
 void progress_bar_main (ClutterContainer *container);
 void slider_main (ClutterContainer *container);
@@ -32,3 +35,4 @@ void scroll_bar_main (ClutterContainer *container);
 void scroll_view_main (ClutterContainer *container);
 void styles_main (ClutterContainer *container);
 void toggle_main (ClutterContainer *container);
+void dialog_main (ClutterContainer *container);
